@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -23,7 +23,7 @@ OF_ASSUME_NONNULL_BEGIN
 
 @interface OFBitSetCharacterSet: OFCharacterSet
 {
-	unsigned char *_bitset;
+	unsigned long *_bitSet;
 	size_t _size;
 }
 @end
