@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -29,8 +29,8 @@
 #ifdef __cplusplus
 extern "C" {
 #endif
-extern uint16_t OFCRC16(uint16_t crc, const void *_Nonnull bytes,
-    size_t length);
+extern uint16_t _OFCRC16(uint16_t crc, const void *_Nonnull bytes,
+    size_t length) OF_VISIBILITY_INTERNAL;
 #ifdef __cplusplus
 }
 #endif
