@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -85,10 +85,8 @@ OFDNSClassParseName(OFString *string)
 
 	if ([string isEqual: @"IN"])
 		DNSClass = OFDNSClassIN;
-	else {
-		DNSClass =
-		    (OFDNSClass)[string unsignedLongLongValueWithBase: 0];
-	}
+	else
+		DNSClass = [string intValueWithBase: 0];
 
 	objc_autoreleasePoolPop(pool);
 
@@ -131,10 +129,8 @@ OFDNSRecordTypeParseName(OFString *string)
 		recordType = OFDNSRecordTypeAll;
 	else if ([string isEqual: @"URI"])
 		recordType = OFDNSRecordTypeURI;
-	else {
-		recordType =
-		    (OFDNSRecordType)[string unsignedLongLongValueWithBase: 0];
-	}
+	else
+		recordType = [string intValueWithBase: 0];
 
 	objc_autoreleasePoolPop(pool);
 
@@ -158,7 +154,7 @@ OFDNSRecordTypeParseName(OFString *string)
 		_recordType = recordType;
 		_TTL = TTL;
 	} @catch (id e) {
-		[self release];
+		objc_release(self);
 		@throw e;
 	}
 
@@ -167,14 +163,14 @@ OFDNSRecordTypeParseName(OFString *string)
 
 - (void)dealloc
 {
-	[_name release];
+	objc_release(_name);
 
 	[super dealloc];
 }
 
 - (id)copy
 {
-	return [self retain];
+	return objc_retain(self);
 }
 
 - (OFString *)description
