@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -241,7 +241,7 @@ encodingForContentType(OFString *contentType)
 		_protocolVersion.minor = 1;
 		_headers = [[OFDictionary alloc] init];
 	} @catch (id e) {
-		[self release];
+		objc_release(self);
 		@throw e;
 	}
 
@@ -250,7 +250,7 @@ encodingForContentType(OFString *contentType)
 
 - (void)dealloc
 {
-	[_headers release];
+	objc_release(_headers);
 
 	[super dealloc];
 }
@@ -275,20 +275,13 @@ encodingForContentType(OFString *contentType)
 {
 	void *pool = objc_autoreleasePoolPush();
 	OFArray *components = [string componentsSeparatedByString: @"."];
-	unsigned long long major, minor;
 	OFHTTPRequestProtocolVersion protocolVersion;
 
 	if (components.count != 2)
 		@throw [OFInvalidFormatException exception];
 
-	major = [components.firstObject unsignedLongLongValue];
-	minor = [components.lastObject unsignedLongLongValue];
-
-	if (major > UCHAR_MAX || minor > UCHAR_MAX)
-		@throw [OFOutOfRangeException exception];
-
-	protocolVersion.major = (unsigned char)major;
-	protocolVersion.minor = (unsigned char)minor;
+	protocolVersion.major = [components.firstObject unsignedCharValue];
+	protocolVersion.minor = [components.lastObject unsignedCharValue];
 
 	self.protocolVersion = protocolVersion;
 
@@ -340,7 +333,7 @@ encodingForContentType(OFString *contentType)
 
 	objc_autoreleasePoolPop(pool);
 
-	return [ret autorelease];
+	return objc_autoreleaseReturnValue(ret);
 }
 
 - (OFString *)description
@@ -361,6 +354,6 @@ encodingForContentType(OFString *contentType)
 
 	objc_autoreleasePoolPop(pool);
 
-	return [ret autorelease];
+	return objc_autoreleaseReturnValue(ret);
 }
 @end
