@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -268,13 +268,14 @@ transformVectors_3DNow(OFMatrix4x4 *self, SEL _cmd, OFVector4D *vectors,
 
 + (OFMatrix4x4 *)identityMatrix
 {
-	return [[[OFMatrix4x4 alloc]
-	    initWithValues: identityValues] autorelease];
+	return objc_autoreleaseReturnValue(
+	    [[OFMatrix4x4 alloc] initWithValues: identityValues]);
 }
 
 + (instancetype)matrixWithValues: (const float [4][4])values
 {
-	return [[[self alloc] initWithValues: values] autorelease];
+	return objc_autoreleaseReturnValue(
+	    [[self alloc] initWithValues: values]);
 }
 
 - (instancetype)init
@@ -318,7 +319,8 @@ transformVectors_3DNow(OFMatrix4x4 *self, SEL _cmd, OFVector4D *vectors,
 
 	for (uint_fast8_t i = 0; i < 4; i++)
 		for (uint_fast8_t j = 0; j < 4; j++)
-			OFHashAddHash(&hash, OFFloatToRawUInt32(_values[i][j]));
+			OFHashAddHash(&hash,
+			    OFBitConvertFloatToUInt32(_values[i][j]));
 
 	OFHashFinalize(&hash);
 
@@ -350,7 +352,7 @@ transformVectors_3DNow(OFMatrix4x4 *self, SEL _cmd, OFVector4D *vectors,
 		{ 0, 0, 0, 1 }
 	    }];
 	[self multiplyWithMatrix: translation];
-	[translation release];
+	objc_release(translation);
 }
 
 - (void)scaleWithVector: (OFVector3D)vector
@@ -363,7 +365,7 @@ transformVectors_3DNow(OFMatrix4x4 *self, SEL _cmd, OFVector4D *vectors,
 		{ 0, 0, 0, 1 }
 	    }];
 	[self multiplyWithMatrix: scale];
-	[scale release];
+	objc_release(scale);
 }
 
 - (OFVector4D)transformedVector: (OFVector4D)vector
