@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -27,22 +27,22 @@
 - (void)setFirstObject: (id)firstObject
 {
 	id old = _firstObject;
-	_firstObject = [firstObject retain];
-	[old release];
+	_firstObject = objc_retain(firstObject);
+	objc_release(old);
 }
 
 - (void)setSecondObject: (id)secondObject
 {
 	id old = _secondObject;
-	_secondObject = [secondObject retain];
-	[old release];
+	_secondObject = objc_retain(secondObject);
+	objc_release(old);
 }
 
 - (void)setThirdObject: (id)thirdObject
 {
 	id old = _thirdObject;
-	_thirdObject = [thirdObject retain];
-	[old release];
+	_thirdObject = objc_retain(thirdObject);
+	objc_release(old);
 }
 
 - (id)copy
