@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -29,16 +29,18 @@
 + (instancetype)notificationWithName: (OFNotificationName)name
 			      object: (id)object
 {
-	return [[[self alloc] initWithName: name object: object] autorelease];
+	return objc_autoreleaseReturnValue([[self alloc] initWithName: name
+							       object: object]);
 }
 
 + (instancetype)notificationWithName: (OFNotificationName)name
 			      object: (id)object
 			    userInfo: (OFDictionary *)userInfo
 {
-	return [[[self alloc] initWithName: name
-				    object: object
-				  userInfo: userInfo] autorelease];
+	return objc_autoreleaseReturnValue(
+	    [[self alloc] initWithName: name
+				object: object
+			      userInfo: userInfo]);
 }
 
 - (instancetype)initWithName: (OFNotificationName)name object: (id)object
@@ -54,10 +56,10 @@
 
 	@try {
 		_name = [name copy];
-		_object = [object retain];
+		_object = objc_retain(object);
 		_userInfo = [userInfo copy];
 	} @catch (id e) {
-		[self release];
+		objc_release(self);
 		@throw e;
 	}
 
@@ -71,15 +73,39 @@
 
 - (void)dealloc
 {
-	[_name release];
-	[_object release];
-	[_userInfo release];
+	objc_release(_name);
+	objc_release(_object);
+	objc_release(_userInfo);
 
 	[super dealloc];
 }
 
 - (id)copy
 {
-	return [self retain];
+	return objc_retain(self);
+}
+
+- (OFString *)description
+{
+	void *pool = objc_autoreleasePoolPush();
+	OFString *object = [[_object description]
+	    stringByReplacingOccurrencesOfString: @"\n"
+				      withString: @"\n\t"];
+	OFString *userInfo = [_userInfo.description
+	    stringByReplacingOccurrencesOfString: @"\n"
+				      withString: @"\n\t"];
+	OFString *ret = [OFString stringWithFormat:
+	    @"<%@:\n"
+	    @"\tName = %@\n"
+	    @"\tObject = %@\n"
+	    @"\tUser info = %@\n"
+	    @">",
+	    self.class, _name, object, userInfo];
+
+	objc_retain(ret);
+
+	objc_autoreleasePoolPop(pool);
+
+	return objc_autoreleaseReturnValue(ret);
 }
 @end
