@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -33,16 +33,11 @@
 #endif
 
 #import "OFObject.h"
+#import "OFApplication.h"
 #import "OFArray.h"
-#ifdef OF_HAVE_ATOMIC_OPS
-# import "OFAtomic.h"
-#endif
 #import "OFLocale.h"
 #import "OFMethodSignature.h"
 #import "OFRunLoop.h"
-#if !defined(OF_HAVE_ATOMIC_OPS) && defined(OF_HAVE_THREADS)
-# import "OFPlainMutex.h"	/* For OFSpinlock */
-#endif
 #import "OFStdIOStream.h"
 #import "OFString.h"
 #import "OFThread.h"
@@ -69,34 +64,32 @@
 
 #ifdef OF_AMIGAOS
 # define Class IntuitionClass
+# define USE_INLINE_STDARG
 # include <proto/exec.h>
+# include <clib/debug_protos.h>
+# define __NOLIBBASE_
+# include <proto/intuition.h>
+# undef __NOLIBBASE_
 # undef Class
 #endif
 
 #ifdef OF_APPLE_RUNTIME
-extern id _Nullable _objc_rootAutorelease(id _Nullable object);
+extern id _objc_rootRetain(id object);
+extern uintptr_t _objc_rootRetainCount(id object);
+extern void _objc_rootRelease(id object);
+extern id _objc_rootAutorelease(id object);
 #endif
 #if defined(OF_HAVE_FORWARDING_TARGET_FOR_SELECTOR)
-extern id OFForward(id, SEL, ...);
-extern struct Stret OFForward_stret(id, SEL, ...);
+extern id _OFForward(id, SEL, ...);
+extern struct Stret _OFForward_stret(id, SEL, ...);
 #else
-# define OFForward OFMethodNotFound
-# define OFForward_stret OFMethodNotFound_stret
+# define _OFForward OFMethodNotFound
+# define _OFForward_stret OFMethodNotFound_stret
 #endif
 
-struct PreIvars {
-#ifdef OF_MSDOS
-	ptrdiff_t offset;
+#ifdef OF_WINDOWS
+static BOOLEAN NTAPI (*RtlGenRandomFuncPtr)(PVOID, ULONG);
 #endif
-	int retainCount;
-#if !defined(OF_HAVE_ATOMIC_OPS) && !defined(OF_AMIGAOS)
-	OFSpinlock retainCountSpinlock;
-#endif
-};
-
-#define PRE_IVARS_ALIGN \
-	OFRoundUpToPowerOf2(sizeof(struct PreIvars), OF_BIGGEST_ALIGNMENT)
-#define PRE_IVARS ((struct PreIvars *)(void *)((char *)self - PRE_IVARS_ALIGN))
 
 static struct {
 	Class isa;
@@ -165,39 +158,24 @@ OFFreeMemory(void *pointer)
 	free(pointer);
 }
 
-#ifdef OF_MSDOS
-/* Unfortunately, DJGPP's memalign() is broken. */
+#if (!defined(HAVE_ARC4RANDOM) && !defined(HAVE_GETRANDOM)) || \
+    defined(OF_WINDOWS)
+static OFOnceControl randomOnceControl = OFOnceControlInitValue;
 
-static void *
-alignedAlloc(size_t size, size_t alignment, ptrdiff_t *offset)
-{
-	char *ptr, *aligned;
-
-	if ((ptr = malloc(size + alignment)) == NULL)
-		return NULL;
-
-	aligned = (char *)OFRoundUpToPowerOf2(alignment, (uintptr_t)ptr);
-	*offset = aligned - ptr;
-
-	return aligned;
-}
-
-static void
-alignedFree(void *ptr, ptrdiff_t offset)
-{
-	if (ptr == NULL)
-		return;
-
-	free((void *)((uintptr_t)ptr - offset));
-}
-#endif
-
-#if !defined(HAVE_ARC4RANDOM) && !defined(HAVE_GETRANDOM)
 static void
 initRandom(void)
 {
 	struct timeval tv;
 
+# ifdef OF_WINDOWS
+	HANDLE handle;
+
+	if ((handle = GetModuleHandleA("advapi32.dll")) != NULL &&
+	    (RtlGenRandomFuncPtr = (BOOLEAN NTAPI (*)(PVOID, ULONG))
+	    GetProcAddress(handle, "SystemFunction036")) != NULL)
+		return;
+# endif
+
 # ifdef HAVE_RANDOM
 	gettimeofday(&tv, NULL);
 	srandom((unsigned)(tv.tv_sec ^ tv.tv_usec));
@@ -220,8 +198,16 @@ OFRandom16(void)
 
 	return buffer;
 #else
-	static OFOnceControl onceControl = OFOnceControlInitValue;
-	OFOnce(&onceControl, initRandom);
+	OFOnce(&randomOnceControl, initRandom);
+
+# ifdef OF_WINDOWS
+	if (RtlGenRandomFuncPtr != NULL) {
+		uint16_t buffer;
+		OFEnsure(RtlGenRandomFuncPtr(&buffer, sizeof(buffer)));
+		return buffer;
+	}
+# endif
+
 # ifdef HAVE_RANDOM
 	return random() & 0xFFFF;
 # else
@@ -242,6 +228,16 @@ OFRandom32(void)
 
 	return buffer;
 #else
+# ifdef OF_WINDOWS
+	OFOnce(&randomOnceControl, initRandom);
+
+	if (RtlGenRandomFuncPtr != NULL) {
+		uint32_t buffer;
+		OFEnsure(RtlGenRandomFuncPtr(&buffer, sizeof(buffer)));
+		return buffer;
+	}
+# endif
+
 	return ((uint32_t)OFRandom16() << 16) | OFRandom16();
 #endif
 }
@@ -262,6 +258,16 @@ OFRandom64(void)
 
 	return buffer;
 #else
+# ifdef OF_WINDOWS
+	OFOnce(&randomOnceControl, initRandom);
+
+	if (RtlGenRandomFuncPtr != NULL) {
+		uint64_t buffer;
+		OFEnsure(RtlGenRandomFuncPtr(&buffer, sizeof(buffer)));
+		return buffer;
+	}
+# endif
+
 	return ((uint64_t)OFRandom32() << 32) | OFRandom32();
 #endif
 }
@@ -287,12 +293,26 @@ typeEncodingForSelector(Class class, SEL selector)
 static void
 uncaughtExceptionHandler(id exception)
 {
+# ifdef OF_AMIGAOS
+#  ifdef OF_HAVE_FILES
+	const char *programName = [OFApplication sharedApplication].programName
+	    .lastPathComponent.UTF8String;
+#  else
+	const char *programName = [OFApplication sharedApplication].programName
+	    .UTF8String;
+#  endif
+# endif
 	OFArray OF_GENERIC(OFValue *) *stackTraceAddresses = nil;
 	OFArray OF_GENERIC(OFString *) *stackTraceSymbols = nil;
 	OFStringEncoding encoding = [OFLocale encoding];
 
+# ifdef OF_AMIGAOS
+	kprintf("%s: Runtime error: Unhandled exception:\n", programName);
+	kprintf("%s: %s\n", programName, [[exception description] UTF8String]);
+# else
 	OFLog(@"Runtime error: Unhandled exception:");
 	OFLog(@"%@", exception);
+# endif
 
 	if ([exception respondsToSelector: @selector(stackTraceAddresses)])
 		stackTraceAddresses = [exception stackTraceAddresses];
@@ -307,8 +327,13 @@ uncaughtExceptionHandler(id exception)
 		if (stackTraceSymbols.count != count)
 			stackTraceSymbols = nil;
 
+# ifdef OF_AMIGAOS
+		kprintf("%s:\n", programName);
+		kprintf("%s: Stack trace:\n", programName);
+# else
 		OFLog(@"");
 		OFLog(@"Stack trace:");
+# endif
 
 		if (stackTraceSymbols != nil) {
 			for (size_t i = 0; i < count; i++) {
@@ -318,19 +343,62 @@ uncaughtExceptionHandler(id exception)
 				    objectAtIndex: i]
 				    cStringWithEncoding: encoding];
 
+# ifdef OF_AMIGAOS
+				kprintf("%s:   %p  %s\n", programName,
+				    address, symbol);
+# else
 				OFLog(@"  %p  %s", address, symbol);
+# endif
 			}
 		} else {
 			for (size_t i = 0; i < count; i++) {
 				void *address = [[stackTraceAddresses
 				    objectAtIndex: i] pointerValue];
 
+# ifdef OF_AMIGAOS
+				kprintf("%s:   %p\n", programName, address);
+# else
 				OFLog(@"  %p", address);
+# endif
 			}
 		}
 	}
 
+# ifdef OF_AMIGAOS
+	struct Library *IntuitionBase;
+#  ifdef OF_AMIGAOS4
+	struct IntuitionIFace *IIntuition;
+#  endif
+	struct EasyStruct easy;
+
+	if ((IntuitionBase = OpenLibrary("intuition.library", 0)) == NULL)
+		abort();
+
+#  ifdef OF_AMIGAOS4
+	if ((IIntuition = (struct IntuitionIFace *)GetInterface(IntuitionBase,
+	    "main", 1, NULL)) == NULL)
+		abort();
+#  endif
+
+	easy.es_StructSize = sizeof(easy);
+	easy.es_Flags = 0;
+	easy.es_Title = (void *)programName;
+	easy.es_TextFormat = (void *)"%s";
+	easy.es_GadgetFormat = (void *)"OK";
+
+	EasyRequest(NULL, &easy, NULL, (ULONG)[[OFString stringWithFormat:
+	    @"Runtime error: Unhandled exception:\n%@", exception] UTF8String]);
+
+#  ifdef OF_AMIGAOS4
+	DropInterface((struct Interface *)IIntuition);
+#  endif
+
+	CloseLibrary(IntuitionBase);
+
+	exit(1);
+# else
 	abort();
+# endif
 }
 #endif
 
@@ -366,31 +434,14 @@ OFAllocObject(Class class, size_t extraSize, size_t extraAlignment,
 {
 	OFObject *instance;
 	size_t instanceSize;
-#ifdef OF_MSDOS
-	ptrdiff_t offset;
-#endif
 
 	instanceSize = class_getInstanceSize(class);
 
 	if OF_UNLIKELY (extraAlignment > 1)
 		extraAlignment = OFRoundUpToPowerOf2(extraAlignment,
-		    PRE_IVARS_ALIGN + instanceSize) -
-		    PRE_IVARS_ALIGN - instanceSize;
+		    instanceSize) - instanceSize;
 
-#if defined(OF_WINDOWS)
-	instance = __mingw_aligned_malloc(PRE_IVARS_ALIGN + instanceSize +
-	    extraAlignment + extraSize, OF_BIGGEST_ALIGNMENT);
-#elif defined(OF_MSDOS)
-	instance = alignedAlloc(PRE_IVARS_ALIGN + instanceSize +
-	    extraAlignment + extraSize, OF_BIGGEST_ALIGNMENT, &offset);
-#elif defined(OF_SOLARIS)
-	if (posix_memalign((void **)&instance, OF_BIGGEST_ALIGNMENT,
-	    PRE_IVARS_ALIGN + instanceSize + extraAlignment + extraSize) != 0)
-		instance = NULL;
-#else
-	instance = malloc(PRE_IVARS_ALIGN + instanceSize +
-	    extraAlignment + extraSize);
-#endif
+	instance = class_createInstance(class, extraAlignment + extraSize);
 
 	if OF_UNLIKELY (instance == nil) {
 		object_setClass((id)&allocFailedException,
@@ -398,46 +449,6 @@ OFAllocObject(Class class, size_t extraSize, size_t extraAlignment,
 		@throw (id)&allocFailedException;
 	}
 
-#ifdef OF_MSDOS
-	((struct PreIvars *)instance)->offset = offset;
-#endif
-	((struct PreIvars *)instance)->retainCount = 1;
-
-#if !defined(OF_HAVE_ATOMIC_OPS) && !defined(OF_AMIGAOS)
-	if OF_UNLIKELY (OFSpinlockNew(
-	    &((struct PreIvars *)instance)->retainCountSpinlock) != 0) {
-# if defined(OF_WINDOWS)
-		__mingw_aligned_free(instance);
-# elif defined(OF_MSDOS)
-		alignedFree(instance, offset);
-# else
-		free(instance);
-# endif
-		@throw [OFInitializationFailedException
-		    exceptionWithClass: class];
-	}
-#endif
-
-	instance = (OFObject *)(void *)((char *)instance + PRE_IVARS_ALIGN);
-
-	memset(instance, 0, instanceSize + extraAlignment + extraSize);
-
-	if (!objc_constructInstance(class, instance)) {
-#if !defined(OF_HAVE_ATOMIC_OPS) && !defined(OF_AMIGAOS)
-		OFSpinlockFree(&((struct PreIvars *)(void *)
-		    ((char *)instance - PRE_IVARS_ALIGN))->retainCountSpinlock);
-#endif
-#if defined(OF_WINDOWS)
-		__mingw_aligned_free((char *)instance - PRE_IVARS_ALIGN);
-#elif defined(OF_MSDOS)
-		alignedFree((char *)instance - PRE_IVARS_ALIGN, offset);
-#else
-		free((char *)instance - PRE_IVARS_ALIGN);
-#endif
-		@throw [OFInitializationFailedException
-		    exceptionWithClass: class];
-	}
-
 	if OF_UNLIKELY (extra != NULL)
 		*extra = (char *)instance + instanceSize + extraAlignment;
 
@@ -451,7 +462,7 @@ _NSPrintForDebugger(id object)
 }
 
 /* References for static linking */
-void
+void OF_VISIBILITY_INTERNAL
 _references_to_categories_of_OFObject(void)
 {
 	_OFObject_KeyValueCoding_reference = 1;
@@ -475,10 +486,10 @@ _references_to_categories_of_OFObject(void)
 	 * already been set, so this is the best we can do.
 	 */
 	if (dlsym(RTLD_DEFAULT, "NSFoundationVersionNumber") == NULL)
-		objc_setForwardHandler((void *)&OFForward,
-		    (void *)&OFForward_stret);
+		objc_setForwardHandler((void *)&_OFForward,
+		    (void *)&_OFForward_stret);
 #else
-	objc_setForwardHandler((IMP)&OFForward, (IMP)&OFForward_stret);
+	objc_setForwardHandler((IMP)&_OFForward, (IMP)&_OFForward_stret);
 #endif
 
 	objc_setEnumerationMutationHandler(enumerationMutationHandler);
@@ -503,7 +514,15 @@ _references_to_categories_of_OFObject(void)
 
 + (instancetype)alloc
 {
-	return OFAllocObject(self, 0, 0, NULL);
+	OFObject *instance = class_createInstance(self, 0);
+
+	if OF_UNLIKELY (instance == nil) {
+		object_setClass((id)&allocFailedException,
+		    [OFAllocFailedException class]);
+		@throw (id)&allocFailedException;
+	}
+
+	return instance;
 }
 
 + (Class)class
@@ -1202,67 +1221,23 @@ _references_to_categories_of_OFObject(void)
 							 object: self];
 }
 
+- (void)_usesRuntimeRR
+{
+}
+
 - (instancetype)retain
 {
-#if defined(OF_HAVE_ATOMIC_OPS)
-	OFAtomicIntIncrease(&PRE_IVARS->retainCount);
-#elif defined(OF_AMIGAOS)
-	/*
-	 * On AmigaOS, we can only have one CPU. As increasing a variable is a
-	 * single instruction on M68K, we don't need Forbid() / Permit() on
-	 * M68K.
-	 */
-# ifndef OF_AMIGAOS_M68K
-	Forbid();
-# endif
-	PRE_IVARS->retainCount++;
-# ifndef OF_AMIGAOS_M68K
-	Permit();
-# endif
-#else
-	OFEnsure(OFSpinlockLock(&PRE_IVARS->retainCountSpinlock) == 0);
-	PRE_IVARS->retainCount++;
-	OFEnsure(OFSpinlockUnlock(&PRE_IVARS->retainCountSpinlock) == 0);
-#endif
-
-	return self;
+	return _objc_rootRetain(self);
 }
 
 - (unsigned int)retainCount
 {
-	OFAssert(PRE_IVARS->retainCount >= 0);
-	return PRE_IVARS->retainCount;
+	return (unsigned int)_objc_rootRetainCount(self);
 }
 
 - (void)release
 {
-#if defined(OF_HAVE_ATOMIC_OPS)
-	OFReleaseMemoryBarrier();
-
-	if (OFAtomicIntDecrease(&PRE_IVARS->retainCount) <= 0) {
-		OFAcquireMemoryBarrier();
-
-		[self dealloc];
-	}
-#elif defined(OF_AMIGAOS)
-	int retainCount;
-
-	Forbid();
-	retainCount = --PRE_IVARS->retainCount;
-	Permit();
-
-	if (retainCount == 0)
-		[self dealloc];
-#else
-	int retainCount;
-
-	OFEnsure(OFSpinlockLock(&PRE_IVARS->retainCountSpinlock) == 0);
-	retainCount = --PRE_IVARS->retainCount;
-	OFEnsure(OFSpinlockUnlock(&PRE_IVARS->retainCountSpinlock) == 0);
-
-	if (retainCount == 0)
-		[self dealloc];
-#endif
+	_objc_rootRelease(self);
 }
 
 - (instancetype)autorelease
@@ -1294,19 +1269,7 @@ _references_to_categories_of_OFObject(void)
 
 - (void)dealloc
 {
-	objc_destructInstance(self);
-
-#if !defined(OF_HAVE_ATOMIC_OPS) && !defined(OF_AMIGAOS)
-	OFSpinlockFree(&PRE_IVARS->retainCountSpinlock);
-#endif
-
-#if defined(OF_WINDOWS)
-	__mingw_aligned_free((char *)self - PRE_IVARS_ALIGN);
-#elif defined(OF_MSDOS)
-	alignedFree((char *)self - PRE_IVARS_ALIGN, PRE_IVARS->offset);
-#else
-	free((char *)self - PRE_IVARS_ALIGN);
-#endif
+	object_dispose(self);
 }
 
 /* Required to use properties with the Apple runtime */
