@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -80,6 +80,9 @@ static OFRunLoop *mainRunLoop = nil;
 # endif
 #endif
 }
+
+- (instancetype)init OF_UNAVAILABLE;
+- (instancetype)initWithMode: (OFRunLoopMode)mode;
 @end
 
 #ifdef OF_HAVE_SOCKETS
@@ -241,6 +244,11 @@ static OFRunLoop *mainRunLoop = nil;
 
 @implementation OFRunLoopState
 - (instancetype)init
+{
+	OF_INVALID_INIT_METHOD
+}
+
+- (instancetype)initWithMode: (OFRunLoopMode)mode
 {
 	self = [super init];
 
@@ -253,6 +261,12 @@ static OFRunLoop *mainRunLoop = nil;
 #ifdef OF_HAVE_SOCKETS
 		_readQueues = [[OFMutableDictionary alloc] init];
 		_writeQueues = [[OFMutableDictionary alloc] init];
+
+		if ([OFKernelEventObserver handlesForeignEvents]) {
+			_kernelEventObserver = [[OFKernelEventObserver alloc]
+			    initWithRunLoopMode: mode];
+			_kernelEventObserver.delegate = self;
+		}
 #endif
 #if defined(OF_HAVE_THREADS)
 		_condition = [[OFCondition alloc] init];
@@ -268,7 +282,7 @@ static OFRunLoop *mainRunLoop = nil;
 # endif
 #endif
 	} @catch (id e) {
-		[self release];
+		objc_release(self);
 		@throw e;
 	}
 
@@ -277,24 +291,24 @@ static OFRunLoop *mainRunLoop = nil;
 
 - (void)dealloc
 {
-	[_timersQueue release];
+	objc_release(_timersQueue);
 #ifdef OF_HAVE_THREADS
-	[_timersQueueMutex release];
+	objc_release(_timersQueueMutex);
 #endif
 #ifdef OF_HAVE_SOCKETS
-	[_kernelEventObserver release];
-	[_readQueues release];
-	[_writeQueues release];
+	objc_release(_kernelEventObserver);
+	objc_release(_readQueues);
+	objc_release(_writeQueues);
 #endif
 #ifdef OF_HAVE_THREADS
-	[_condition release];
+	objc_release(_condition);
 #endif
 #ifdef OF_AMIGAOS
-	[_execSignals release];
-	[_execSignalsTargets release];
-	[_execSignalsSelectors release];
+	objc_release(_execSignals);
+	objc_release(_execSignalsTargets);
+	objc_release(_execSignalsSelectors);
 # ifdef OF_HAVE_THREADS
-	[_execSignalsMutex release];
+	objc_release(_execSignalsMutex);
 # endif
 #endif
 
@@ -309,7 +323,7 @@ static OFRunLoop *mainRunLoop = nil;
 	 * handler called -[cancelAsyncRequests].
 	 */
 	OFList OF_GENERIC(OF_KINDOF(OFRunLoopReadQueueItem *)) *queue =
-	    [[_readQueues objectForKey: object] retain];
+	    objc_retain([_readQueues objectForKey: object]);
 
 	OFAssert(queue != nil);
 
@@ -329,8 +343,8 @@ static OFRunLoop *mainRunLoop = nil;
 				 * this is that the target might call
 				 * -[cancelAsyncRequests] in its dealloc.
 				 */
-				[[OFListItemObject(listItem) retain]
-				    autorelease];
+				objc_retainAutorelease(
+				    OFListItemObject(listItem));
 
 				[queue removeListItem: listItem];
 
@@ -343,7 +357,7 @@ static OFRunLoop *mainRunLoop = nil;
 			}
 		}
 	} @finally {
-		[queue release];
+		objc_release(queue);
 	}
 }
 
@@ -353,7 +367,7 @@ static OFRunLoop *mainRunLoop = nil;
 	 * Retain the queue so that it doesn't disappear from us because the
 	 * handler called -[cancelAsyncRequests].
 	 */
-	OFList *queue = [[_writeQueues objectForKey: object] retain];
+	OFList *queue = objc_retain([_writeQueues objectForKey: object]);
 
 	OFAssert(queue != nil);
 
@@ -373,8 +387,8 @@ static OFRunLoop *mainRunLoop = nil;
 				 * this is that the target might call
 				 * -[cancelAsyncRequests] in its dealloc.
 				 */
-				[[OFListItemObject(listItem) retain]
-				    autorelease];
+				objc_retainAutorelease(
+				    OFListItemObject(listItem));
 
 				[queue removeListItem: listItem];
 
@@ -387,7 +401,7 @@ static OFRunLoop *mainRunLoop = nil;
 			}
 		}
 	} @finally {
-		[queue release];
+		objc_release(queue);
 	}
 }
 #endif
@@ -412,9 +426,9 @@ static OFRunLoop *mainRunLoop = nil;
 		 * Create copies, so that signal handlers are allowed to modify
 		 * signals.
 		 */
-		signals = [[_execSignals copy] autorelease];
-		targets = [[_execSignalsTargets copy] autorelease];
-		selectors = [[_execSignalsSelectors copy] autorelease];
+		signals = objc_autorelease([_execSignals copy]);
+		targets = objc_autorelease([_execSignalsTargets copy]);
+		selectors = objc_autorelease([_execSignalsSelectors copy]);
 # ifdef OF_HAVE_THREADS
 	} @finally {
 		[_execSignalsMutex unlock];
@@ -451,7 +465,7 @@ static OFRunLoop *mainRunLoop = nil;
 
 - (void)dealloc
 {
-	[_delegate release];
+	objc_release(_delegate);
 
 	[super dealloc];
 }
@@ -491,7 +505,7 @@ static OFRunLoop *mainRunLoop = nil;
 # ifdef OF_HAVE_BLOCKS
 - (void)dealloc
 {
-	[_handler release];
+	objc_release(_handler);
 
 	[super dealloc];
 }
@@ -547,7 +561,7 @@ static OFRunLoop *mainRunLoop = nil;
 # ifdef OF_HAVE_BLOCKS
 - (void)dealloc
 {
-	[_handler release];
+	objc_release(_handler);
 
 	[super dealloc];
 }
@@ -590,7 +604,7 @@ static OFRunLoop *mainRunLoop = nil;
 # ifdef OF_HAVE_BLOCKS
 - (void)dealloc
 {
-	[_handler release];
+	objc_release(_handler);
 
 	[super dealloc];
 }
@@ -633,7 +647,7 @@ static OFRunLoop *mainRunLoop = nil;
 # ifdef OF_HAVE_BLOCKS
 - (void)dealloc
 {
-	[_handler release];
+	objc_release(_handler);
 
 	[super dealloc];
 }
@@ -655,7 +669,8 @@ static OFRunLoop *mainRunLoop = nil;
 	} @catch (OFWriteFailedException *e) {
 		length = e.bytesWritten;
 
-		if (e.errNo != EWOULDBLOCK && e.errNo != EAGAIN)
+		if (e.errNo != EWOULDBLOCK && e.errNo != EAGAIN &&
+		    (e.errNo != 0 || length <= 0))
 			exception = e;
 	} @catch (id e) {
 		length = 0;
@@ -677,7 +692,7 @@ static OFRunLoop *mainRunLoop = nil;
 
 		oldData = _data;
 		_data = [newData copy];
-		[oldData release];
+		objc_release(oldData);
 
 		_writtenLength = 0;
 		return true;
@@ -697,7 +712,7 @@ static OFRunLoop *mainRunLoop = nil;
 
 		oldData = _data;
 		_data = [newData copy];
-		[oldData release];
+		objc_release(oldData);
 
 		_writtenLength = 0;
 		return true;
@@ -709,9 +724,9 @@ static OFRunLoop *mainRunLoop = nil;
 - (void)dealloc
 {
 # ifdef OF_HAVE_BLOCKS
-	[_handler release];
+	objc_release(_handler);
 # endif
-	[_data release];
+	objc_release(_data);
 
 	[super dealloc];
 }
@@ -755,7 +770,7 @@ static OFRunLoop *mainRunLoop = nil;
 
 		oldString = _string;
 		_string = [newString copy];
-		[oldString release];
+		objc_release(oldString);
 
 		_writtenLength = 0;
 		return true;
@@ -776,7 +791,7 @@ static OFRunLoop *mainRunLoop = nil;
 
 		oldString = _string;
 		_string = [newString copy];
-		[oldString release];
+		objc_release(oldString);
 
 		_writtenLength = 0;
 		return true;
@@ -787,9 +802,9 @@ static OFRunLoop *mainRunLoop = nil;
 
 - (void)dealloc
 {
-	[_string release];
+	objc_release(_string);
 # ifdef OF_HAVE_BLOCKS
-	[_handler release];
+	objc_release(_handler);
 # endif
 
 	[super dealloc];
@@ -874,7 +889,7 @@ static OFRunLoop *mainRunLoop = nil;
 # ifdef OF_HAVE_BLOCKS
 - (void)dealloc
 {
-	[_handler release];
+	objc_release(_handler);
 
 	[super dealloc];
 }
@@ -919,7 +934,7 @@ static OFRunLoop *mainRunLoop = nil;
 # ifdef OF_HAVE_BLOCKS
 - (void)dealloc
 {
-	[_handler release];
+	objc_release(_handler);
 
 	[super dealloc];
 }
@@ -949,7 +964,7 @@ static OFRunLoop *mainRunLoop = nil;
 
 		oldData = _data;
 		_data = [newData copy];
-		[oldData release];
+		objc_release(oldData);
 
 		return true;
 	} else {
@@ -968,7 +983,7 @@ static OFRunLoop *mainRunLoop = nil;
 
 		oldData = _data;
 		_data = [newData copy];
-		[oldData release];
+		objc_release(oldData);
 
 		return true;
 # ifdef OF_HAVE_BLOCKS
@@ -979,9 +994,9 @@ static OFRunLoop *mainRunLoop = nil;
 - (void)dealloc
 {
 # ifdef OF_HAVE_BLOCKS
-	[_handler release];
+	objc_release(_handler);
 # endif
-	[_data release];
+	objc_release(_data);
 
 	[super dealloc];
 }
@@ -1021,7 +1036,7 @@ static OFRunLoop *mainRunLoop = nil;
 # ifdef OF_HAVE_BLOCKS
 - (void)dealloc
 {
-	[_handler release];
+	objc_release(_handler);
 
 	[super dealloc];
 }
@@ -1050,7 +1065,7 @@ static OFRunLoop *mainRunLoop = nil;
 
 		oldData = _data;
 		_data = [newData copy];
-		[oldData release];
+		objc_release(oldData);
 
 		return true;
 	} else {
@@ -1068,7 +1083,7 @@ static OFRunLoop *mainRunLoop = nil;
 
 		oldData = _data;
 		_data = [newData copy];
-		[oldData release];
+		objc_release(oldData);
 
 		return true;
 # ifdef OF_HAVE_BLOCKS
@@ -1079,9 +1094,9 @@ static OFRunLoop *mainRunLoop = nil;
 - (void)dealloc
 {
 # ifdef OF_HAVE_BLOCKS
-	[_handler release];
+	objc_release(_handler);
 # endif
-	[_data release];
+	objc_release(_data);
 
 	[super dealloc];
 }
@@ -1126,7 +1141,7 @@ static OFRunLoop *mainRunLoop = nil;
 # ifdef OF_HAVE_BLOCKS
 - (void)dealloc
 {
-	[_handler release];
+	objc_release(_handler);
 
 	[super dealloc];
 }
@@ -1156,7 +1171,7 @@ static OFRunLoop *mainRunLoop = nil;
 
 		oldData = _data;
 		_data = [newData copy];
-		[oldData release];
+		objc_release(oldData);
 
 		return true;
 	} else {
@@ -1175,7 +1190,7 @@ static OFRunLoop *mainRunLoop = nil;
 
 		oldData = _data;
 		_data = [newData copy];
-		[oldData release];
+		objc_release(oldData);
 
 		return true;
 #  ifdef OF_HAVE_BLOCKS
@@ -1186,10 +1201,10 @@ static OFRunLoop *mainRunLoop = nil;
 - (void)dealloc
 {
 # ifdef OF_HAVE_BLOCKS
-	[_handler release];
+	objc_release(_handler);
 # endif
-	[_data release];
-	[_info release];
+	objc_release(_data);
+	objc_release(_info);
 
 	[super dealloc];
 }
@@ -1216,7 +1231,7 @@ static OFRunLoop *mainRunLoop = nil;
 
 + (void)of_setMainRunLoop: (OFRunLoop *)runLoop
 {
-	mainRunLoop = [runLoop retain];
+	mainRunLoop = objc_retain(runLoop);
 }
 
 static OFRunLoopState *
@@ -1232,19 +1247,33 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 		state = [self->_states objectForKey: mode];
 
 		if (create && state == nil) {
-			state = [[OFRunLoopState alloc] init];
+			state = [[OFRunLoopState alloc] initWithMode: mode];
 			@try {
 				[self->_states setObject: state forKey: mode];
 			} @finally {
-				[state release];
+				objc_release(state);
 			}
 		}
 
 #ifdef OF_HAVE_SOCKETS
 		if (createObserver && state->_kernelEventObserver == nil) {
 			state->_kernelEventObserver =
-			    [[OFKernelEventObserver alloc] init];
+			    [[OFKernelEventObserver alloc]
+			    initWithRunLoopMode: mode];
 			state->_kernelEventObserver.delegate = state;
+# ifdef OF_AMIGAOS
+#  ifdef OF_HAVE_THREADS
+			[state->_execSignalsMutex lock];
+			@try {
+#  endif
+				state->_kernelEventObserver.execSignalMask =
+				    state->_execSignalMask;
+#  ifdef OF_HAVE_THREADS
+			} @finally {
+				[state->_execSignalsMutex unlock];
+			}
+#  endif
+# endif
 		}
 #endif
 #ifdef OF_HAVE_THREADS
@@ -1273,7 +1302,7 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 		[state->_kernelEventObserver				 \
 		    addObjectForReading: object];			 \
 									 \
-	queueItem = [[[type alloc] init] autorelease];
+	queueItem = objc_autorelease([[type alloc] init]);
 # define NEW_WRITE(type, object, mode)					 \
 	void *pool = objc_autoreleasePoolPush();			 \
 	OFRunLoop *runLoop = [self currentRunLoop];			 \
@@ -1290,7 +1319,7 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 		[state->_kernelEventObserver				 \
 		    addObjectForWriting: object];			 \
 									 \
-	queueItem = [[[type alloc] init] autorelease];
+	queueItem = objc_autorelease([[type alloc] init]);
 #define QUEUE_ITEM							 \
 	[queue appendObject: queueItem];				 \
 									 \
@@ -1308,7 +1337,7 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 {
 	NEW_READ(OFRunLoopReadQueueItem, stream, mode)
 
-	queueItem->_delegate = [delegate retain];
+	queueItem->_delegate = objc_retain(delegate);
 # ifdef OF_HAVE_BLOCKS
 	queueItem->_handler = [handler copy];
 # endif
@@ -1330,7 +1359,7 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 {
 	NEW_READ(OFRunLoopExactReadQueueItem, stream, mode)
 
-	queueItem->_delegate = [delegate retain];
+	queueItem->_delegate = objc_retain(delegate);
 # ifdef OF_HAVE_BLOCKS
 	queueItem->_handler = [handler copy];
 # endif
@@ -1351,7 +1380,7 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 {
 	NEW_READ(OFRunLoopReadStringQueueItem, stream, mode)
 
-	queueItem->_delegate = [delegate retain];
+	queueItem->_delegate = objc_retain(delegate);
 # ifdef OF_HAVE_BLOCKS
 	queueItem->_handler = [handler copy];
 # endif
@@ -1371,7 +1400,7 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 {
 	NEW_READ(OFRunLoopReadLineQueueItem, stream, mode)
 
-	queueItem->_delegate = [delegate retain];
+	queueItem->_delegate = objc_retain(delegate);
 # ifdef OF_HAVE_BLOCKS
 	queueItem->_handler = [handler copy];
 # endif
@@ -1391,7 +1420,7 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 {
 	NEW_WRITE(OFRunLoopWriteDataQueueItem, stream, mode)
 
-	queueItem->_delegate = [delegate retain];
+	queueItem->_delegate = objc_retain(delegate);
 # ifdef OF_HAVE_BLOCKS
 	queueItem->_handler = [handler copy];
 # endif
@@ -1412,7 +1441,7 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 {
 	NEW_WRITE(OFRunLoopWriteStringQueueItem, stream, mode)
 
-	queueItem->_delegate = [delegate retain];
+	queueItem->_delegate = objc_retain(delegate);
 # ifdef OF_HAVE_BLOCKS
 	queueItem->_handler = [handler copy];
 # endif
@@ -1429,7 +1458,7 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 {
 	NEW_WRITE(OFRunLoopConnectQueueItem, sock, mode)
 
-	queueItem->_delegate = [delegate retain];
+	queueItem->_delegate = objc_retain(delegate);
 
 	QUEUE_ITEM
 }
@@ -1442,7 +1471,7 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 {
 	NEW_READ(OFRunLoopAcceptQueueItem, sock, mode)
 
-	queueItem->_delegate = [delegate retain];
+	queueItem->_delegate = objc_retain(delegate);
 # ifdef OF_HAVE_BLOCKS
 	queueItem->_handler = [handler copy];
 # endif
@@ -1461,7 +1490,7 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 {
 	NEW_READ(OFRunLoopDatagramReceiveQueueItem, sock, mode)
 
-	queueItem->_delegate = [delegate retain];
+	queueItem->_delegate = objc_retain(delegate);
 # ifdef OF_HAVE_BLOCKS
 	queueItem->_handler = [handler copy];
 # endif
@@ -1482,7 +1511,7 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 {
 	NEW_WRITE(OFRunLoopDatagramSendQueueItem, sock, mode)
 
-	queueItem->_delegate = [delegate retain];
+	queueItem->_delegate = objc_retain(delegate);
 # ifdef OF_HAVE_BLOCKS
 	queueItem->_handler = [handler copy];
 # endif
@@ -1504,7 +1533,7 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 {
 	NEW_READ(OFRunLoopPacketReceiveQueueItem, sock, mode)
 
-	queueItem->_delegate = [delegate retain];
+	queueItem->_delegate = objc_retain(delegate);
 # ifdef OF_HAVE_BLOCKS
 	queueItem->_handler = [handler copy];
 # endif
@@ -1524,7 +1553,7 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 {
 	NEW_WRITE(OFRunLoopPacketSendQueueItem, sock, mode)
 
-	queueItem->_delegate = [delegate retain];
+	queueItem->_delegate = objc_retain(delegate);
 # ifdef OF_HAVE_BLOCKS
 	queueItem->_handler = [handler copy];
 # endif
@@ -1546,7 +1575,7 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 {
 	NEW_READ(OFRunLoopSCTPReceiveQueueItem, sock, mode)
 
-	queueItem->_delegate = [delegate retain];
+	queueItem->_delegate = objc_retain(delegate);
 #  ifdef OF_HAVE_BLOCKS
 	queueItem->_handler = [handler copy];
 #  endif
@@ -1567,7 +1596,7 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 {
 	NEW_WRITE(OFRunLoopSCTPSendQueueItem, sock, mode)
 
-	queueItem->_delegate = [delegate retain];
+	queueItem->_delegate = objc_retain(delegate);
 # ifdef OF_HAVE_BLOCKS
 	queueItem->_handler = [handler copy];
 # endif
@@ -1630,18 +1659,19 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 
 		_states = [[OFMutableDictionary alloc] init];
 
-		state = [[OFRunLoopState alloc] init];
+		state = [[OFRunLoopState alloc]
+		    initWithMode: OFDefaultRunLoopMode];
 		@try {
 			[_states setObject: state forKey: OFDefaultRunLoopMode];
 		} @finally {
-			[state release];
+			objc_release(state);
 		}
 
 #ifdef OF_HAVE_THREADS
 		_statesMutex = [[OFMutex alloc] init];
 #endif
 	} @catch (id e) {
-		[self release];
+		objc_release(self);
 		@throw e;
 	}
 
@@ -1650,9 +1680,9 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 
 - (void)dealloc
 {
-	[_states release];
+	objc_release(_states);
 #ifdef OF_HAVE_THREADS
-	[_statesMutex release];
+	objc_release(_statesMutex);
 #endif
 
 	[super dealloc];
@@ -1870,8 +1900,8 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 				if (listItem != NULL &&
 				    [OFListItemObject(listItem) fireDate]
 				    .timeIntervalSinceNow <= 0) {
-					timer = [[OFListItemObject(listItem)
-					    retain] autorelease];
+					timer = objc_retainAutorelease(
+					    OFListItemObject(listItem));
 
 					[state->_timersQueue
 					    removeListItem: listItem];
@@ -1923,20 +1953,23 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 
 #ifdef OF_HAVE_SOCKETS
 			if (state->_kernelEventObserver != nil) {
-				@try {
-					[state->_kernelEventObserver
-					    observeForTimeInterval: timeout];
-				} @catch (OFObserveKernelEventsFailedException
-				    *e) {
-					if (e.errNo != EINTR)
-						@throw e;
-				}
+				[state->_kernelEventObserver
+				    observeForTimeInterval: timeout];
 			} else {
 #endif
 #ifdef OF_HAVE_THREADS
 				[state->_condition lock];
 # ifdef OF_AMIGAOS
-				signalMask = state->_execSignalMask;
+#  ifdef OF_HAVE_THREADS
+				[state->_execSignalsMutex lock];
+				@try {
+#  endif
+					signalMask = state->_execSignalMask;
+#  ifdef OF_HAVE_THREADS
+				} @finally {
+					[state->_execSignalsMutex unlock];
+				}
+#  endif
 				[state->_condition
 				    waitForTimeInterval: timeout
 					   orExecSignal: &signalMask];
@@ -1961,20 +1994,23 @@ stateForMode(OFRunLoop *self, OFRunLoopMode mode, bool create,
 			 * another thread, it cancels the observe.
 			 */
 #ifdef OF_HAVE_SOCKETS
-			if (state->_kernelEventObserver != nil) {
-				@try {
-					[state->_kernelEventObserver observe];
-				} @catch (OFObserveKernelEventsFailedException
-				    *e) {
-					if (e.errNo != EINTR)
-						@throw e;
-				}
-			} else {
+			if (state->_kernelEventObserver != nil)
+				[state->_kernelEventObserver observe];
+			else {
 #endif
 #ifdef OF_HAVE_THREADS
 				[state->_condition lock];
 # ifdef OF_AMIGAOS
-				signalMask = state->_execSignalMask;
+#  ifdef OF_HAVE_THREADS
+				[state->_execSignalsMutex lock];
+				@try {
+#  endif
+					signalMask = state->_execSignalMask;
+#  ifdef OF_HAVE_THREADS
+				} @finally {
+					[state->_execSignalsMutex unlock];
+				}
+#  endif
 				[state->_condition
 				    waitForConditionOrExecSignal: &signalMask];
 				if (signalMask != 0)
