@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -20,6 +20,7 @@
 #include "config.h"
 
 #include <errno.h>
+#include <math.h>
 
 #include "unistd_wrapper.h"
 
@@ -35,8 +36,9 @@
 #import "OFApplication.h"
 #import "OFColor.h"
 #import "OFDate.h"
+#import "OFDictionary.h"
 #ifdef OF_WINDOWS
-# import "OFWin32ConsoleStdIOStream.h"
+# import "platform/Windows/OFWin32ConsoleStdIOStream.h"
 #endif
 
 #import "OFInitializationFailedException.h"
@@ -64,7 +66,9 @@
 
 #ifdef OF_WII
 # define asm __asm__
+# define id ogc_id
 # include <gccore.h>
+# undef id
 # undef asm
 #endif
 
@@ -137,8 +141,8 @@ OFLogV(OFConstantString *format, va_list arguments)
 	if (me == nil)
 		me = @"?";
 
-	msg = [[[OFString alloc] initWithFormat: format
-				      arguments: arguments] autorelease];
+	msg = objc_autorelease([[OFString alloc] initWithFormat: format
+						      arguments: arguments]);
 
 	[OFStdErr writeFormat: @"[%@.%03d %@(%d)] %@\n", dateString,
 			       date.microsecond / 1000, me, getpid(), msg];
@@ -148,39 +152,41 @@ OFLogV(OFConstantString *format, va_list arguments)
 
 #ifdef OF_MSDOS
 int
-colorToMSDOS(OFColor *color)
+colorToMSDOS(OFColor *color, int default_)
 {
-	if ([color isEqual: [OFColor black]])
+	if (color == nil)
+		return default_;
+	if (color == [OFColor black])
 		return BLACK;
-	if ([color isEqual: [OFColor navy]])
+	if (color == [OFColor navy])
 		return BLUE;
-	if ([color isEqual: [OFColor green]])
+	if (color == [OFColor green])
 		return GREEN;
-	if ([color isEqual: [OFColor teal]])
+	if (color == [OFColor teal])
 		return CYAN;
-	if ([color isEqual: [OFColor maroon]])
+	if (color == [OFColor maroon])
 		return RED;
-	if ([color isEqual: [OFColor purple]])
+	if (color == [OFColor purple])
 		return MAGENTA;
-	if ([color isEqual: [OFColor olive]])
+	if (color == [OFColor olive])
 		return BROWN;
-	if ([color isEqual: [OFColor silver]])
+	if (color == [OFColor silver])
 		return LIGHTGRAY;
-	if ([color isEqual: [OFColor gray]])
+	if (color == [OFColor gray])
 		return DARKGRAY;
-	if ([color isEqual: [OFColor blue]])
+	if (color == [OFColor blue])
 		return LIGHTBLUE;
-	if ([color isEqual: [OFColor lime]])
+	if (color == [OFColor lime])
 		return LIGHTGREEN;
-	if ([color isEqual: [OFColor aqua]])
+	if (color == [OFColor aqua])
 		return LIGHTCYAN;
-	if ([color isEqual: [OFColor red]])
+	if (color == [OFColor red])
 		return LIGHTRED;
-	if ([color isEqual: [OFColor fuchsia]])
+	if (color == [OFColor fuchsia])
 		return LIGHTMAGENTA;
-	if ([color isEqual: [OFColor yellow]])
+	if (color == [OFColor yellow])
 		return YELLOW;
-	if ([color isEqual: [OFColor white]])
+	if (color == [OFColor white])
 		return WHITE;
 
 	return -1;
@@ -189,41 +195,127 @@ colorToMSDOS(OFColor *color)
 static int
 colorToANSI(OFColor *color)
 {
-	if ([color isEqual: [OFColor black]])
+	if (color == nil)
+		return 39;
+	if (color == [OFColor black])
 		return 30;
-	if ([color isEqual: [OFColor maroon]])
+	if (color == [OFColor maroon])
 		return 31;
-	if ([color isEqual: [OFColor green]])
+	if (color == [OFColor green])
 		return 32;
-	if ([color isEqual: [OFColor olive]])
+	if (color == [OFColor olive])
 		return 33;
-	if ([color isEqual: [OFColor navy]])
+	if (color == [OFColor navy])
 		return 34;
-	if ([color isEqual: [OFColor purple]])
+	if (color == [OFColor purple])
 		return 35;
-	if ([color isEqual: [OFColor teal]])
+	if (color == [OFColor teal])
 		return 36;
-	if ([color isEqual: [OFColor silver]])
+	if (color == [OFColor silver])
 		return 37;
-	if ([color isEqual: [OFColor gray]])
+	if (color == [OFColor gray])
 		return 90;
-	if ([color isEqual: [OFColor red]])
+	if (color == [OFColor red])
 		return 91;
-	if ([color isEqual: [OFColor lime]])
+	if (color == [OFColor lime])
 		return 92;
-	if ([color isEqual: [OFColor yellow]])
+	if (color == [OFColor yellow])
 		return 93;
-	if ([color isEqual: [OFColor blue]])
+	if (color == [OFColor blue])
 		return 94;
-	if ([color isEqual: [OFColor fuchsia]])
+	if (color == [OFColor fuchsia])
 		return 95;
-	if ([color isEqual: [OFColor aqua]])
+	if (color == [OFColor aqua])
 		return 96;
-	if ([color isEqual: [OFColor white]])
+	if (color == [OFColor white])
 		return 97;
 
 	return -1;
 }
+
+static unsigned int
+channelTo6Values(uint8_t channel)
+{
+	if (channel >= 235)
+		return 5;
+	if (channel >= 195)
+		return 4;
+	if (channel >= 155)
+		return 3;
+	if (channel >= 115)
+		return 2;
+	if (channel >= 75)
+		return 1;
+
+	return 0;
+}
+
+static unsigned int
+colorTo256Color(uint8_t red, uint8_t green, uint8_t blue)
+{
+	int redIndex, greenIndex, blueIndex;
+
+	if (red == green && green == blue) {
+		if (red >= 244)
+			return 231;
+		if (red >= 234)
+			return 255;
+		if (red >= 224)
+			return 254;
+		if (red >= 214)
+			return 253;
+		if (red >= 204)
+			return 252;
+		if (red >= 194)
+			return 251;
+		if (red >= 184)
+			return 250;
+		if (red >= 174)
+			return 249;
+		if (red >= 164)
+			return 248;
+		if (red >= 154)
+			return 247;
+		if (red >= 144)
+			return 246;
+		if (red >= 134)
+			return 245;
+		if (red >= 124)
+			return 244;
+		if (red >= 114)
+			return 243;
+		if (red >= 104)
+			return 242;
+		if (red >= 94)
+			return 241;
+		if (red >= 84)
+			return 240;
+		if (red >= 74)
+			return 239;
+		if (red >= 64)
+			return 238;
+		if (red >= 54)
+			return 237;
+		if (red >= 44)
+			return 236;
+		if (red >= 34)
+			return 235;
+		if (red >= 24)
+			return 234;
+		if (red >= 14)
+			return 233;
+		if (red >= 4)
+			return 232;
+
+		return 16;
+	}
+
+	redIndex = channelTo6Values(red);
+	greenIndex = channelTo6Values(green);
+	blueIndex = channelTo6Values(blue);
+
+	return 16 + 36 * redIndex + 6 * greenIndex + blueIndex;
+}
 #endif
 
 @implementation OFStdIOStream
@@ -330,13 +422,19 @@ colorToANSI(OFColor *color)
 
 	_handle = handle;
 	_closable = closable;
+	_colors = 16;
+	_cursorVisible = true;
 
 	return self;
 }
 #elif defined(OF_WII_U)
 - (instancetype)of_init
 {
-	return [super init];
+	self = [super init];
+
+	_colors = -1;
+
+	return self;
 }
 #else
 - (instancetype)of_initWithFileDescriptor: (int)fd
@@ -344,6 +442,12 @@ colorToANSI(OFColor *color)
 	self = [super init];
 
 	_fd = fd;
+# ifdef OF_MSDOS
+	_colors = 16;
+# else
+	_colors = -1;
+# endif
+	_cursorVisible = self.hasTerminal;
 
 	return self;
 }
@@ -561,8 +665,10 @@ colorToANSI(OFColor *color)
 
 - (bool)hasTerminal
 {
-#if defined(OF_WII) || defined(OF_NINTENDO_DS) || defined(OF_NINTENDO_3DS) || \
-    defined(OF_NINTENDO_SWITCH)
+#if defined(OF_AMIGAOS)
+	return IsInteractive(_handle);
+#elif defined(OF_WII) || defined(OF_NINTENDO_DS) || \
+    defined(OF_NINTENDO_3DS) || defined(OF_NINTENDO_SWITCH)
 	return true;
 #elif defined(HAVE_ISATTY) && !defined(OF_WII_U)
 	return isatty(_fd);
@@ -613,46 +719,243 @@ colorToANSI(OFColor *color)
 #endif
 }
 
+- (int)colors
+{
+	void *pool;
+	OFDictionary OF_GENERIC(OFString *, OFString *) *environment;
+	OFString *var;
+
+	if (_colors != -1)
+		return _colors;
+
+	if (!self.hasTerminal)
+		return -1;
+
+	pool = objc_autoreleasePoolPush();
+	environment = [OFApplication environment];
+
+	var = [environment objectForKey: @"COLORTERM"];
+	if ([var isEqual: @"24bit"] || [var isEqual: @"truecolor"] ||
+	    [var isEqual: @"16777216"]) {
+		_colors = 16777216;
+		objc_autoreleasePoolPop(pool);
+		return _colors;
+	}
+
+	var = [environment objectForKey: @"TERM"];
+	if ([var hasSuffix: @"-256color"]) {
+		_colors = 256;
+		objc_autoreleasePoolPop(pool);
+		return _colors;
+	}
+
+	_colors = 16;
+	objc_autoreleasePoolPop(pool);
+	return _colors;
+}
+
+- (OFColor *)foregroundColor
+{
+	return _foregroundColor;
+}
+
 - (void)setForegroundColor: (OFColor *)color
 {
 	int code;
 
+	if (color == _foregroundColor)
+		return;
+
 	if (!self.hasTerminal)
 		return;
 
 #ifdef OF_MSDOS
-	if ((code = colorToMSDOS(color)) == -1)
+	if ((code = colorToMSDOS(color, LIGHTGRAY)) == -1)
 		return;
 
 	textcolor(code);
 #else
-	if ((code = colorToANSI(color)) == -1)
-		return;
+	if ((code = colorToANSI(color)) != -1)
+		[self writeFormat: @"\033[%um", code];
+	else if (self.colors >= 256) {
+		float red, green, blue, alpha;
+		uint8_t redInt, greenInt, blueInt;
 
-	[self writeFormat: @"\033[%um", code];
+		[color getRed: &red green: &green blue: &blue alpha: &alpha];
+		if (alpha != 1 || red < 0 || red > 1 || green < 0 ||
+		    green > 1 || blue < 0 || blue > 1)
+			return;
+
+		redInt = roundf(red * 255);
+		greenInt = roundf(green * 255);
+		blueInt = roundf(blue * 255);
+
+		if (self.colors == 16777216)
+			[self writeFormat: @"\033[38;2;%u;%u;%um",
+					   redInt, greenInt, blueInt];
+		else
+			[self writeFormat: @"\033[38;5;%um",
+					   colorTo256Color(redInt, greenInt,
+					       blueInt)];
+	}
 #endif
+
+	objc_release(_foregroundColor);
+	_foregroundColor = objc_retain(color);
+}
+
+- (OFColor *)backgroundColor
+{
+	return _backgroundColor;
 }
 
 - (void)setBackgroundColor: (OFColor *)color
 {
 	int code;
 
+	if (color == _backgroundColor)
+		return;
+
 	if (!self.hasTerminal)
 		return;
 
 #ifdef OF_MSDOS
-	if ((code = colorToMSDOS(color)) == -1)
+	if ((code = colorToMSDOS(color, BLACK)) == -1)
 		return;
 
 	textbackground(code);
 #else
-	if ((code = colorToANSI(color)) == -1)
+	if ((code = colorToANSI(color)) != -1)
+		[self writeFormat: @"\033[%um", code + 10];
+	else {
+		float red, green, blue, alpha;
+		uint8_t redInt, greenInt, blueInt;
+
+		[color getRed: &red green: &green blue: &blue alpha: &alpha];
+		if (alpha != 1 || red < 0 || red > 1 || green < 0 ||
+		    green > 1 || blue < 0 || blue > 1)
+			return;
+
+		redInt = roundf(red * 255);
+		greenInt = roundf(green * 255);
+		blueInt = roundf(blue * 255);
+
+		if (self.colors == 16777216)
+			[self writeFormat: @"\033[48;2;%u;%u;%um",
+					   redInt, greenInt, blueInt];
+		else
+			[self writeFormat: @"\033[48;5;%um",
+					   colorTo256Color(redInt, greenInt,
+					       blueInt)];
+	}
+#endif
+
+	objc_release(_backgroundColor);
+	_backgroundColor = objc_retain(color);
+}
+
+- (bool)isBold
+{
+	return _bold;
+}
+
+- (void)setBold: (bool)bold
+{
+#ifndef OF_MSDOS
+	if (!self.hasTerminal)
 		return;
 
-	[self writeFormat: @"\033[%um", code + 10];
+	if (bold == _bold)
+		return;
+
+	[self writeString: (bold ? @"\033[1m" : @"\033[22m")];
+
+	_bold = bold;
 #endif
 }
 
+- (bool)isItalic
+{
+	return _italic;
+}
+
+- (void)setItalic: (bool)italic
+{
+#ifndef OF_MSDOS
+	if (!self.hasTerminal)
+		return;
+
+	if (italic == _italic)
+		return;
+
+	[self writeString: (italic ? @"\033[3m" : @"\033[23m")];
+
+	_italic = italic;
+#endif
+}
+
+- (bool)isUnderlined
+{
+	return _underlined;
+}
+
+- (void)setUnderlined: (bool)underlined
+{
+#ifndef OF_MSDOS
+	if (!self.hasTerminal)
+		return;
+
+	if (underlined == _underlined)
+		return;
+
+	[self writeString: (underlined ? @"\033[4m" : @"\033[24m")];
+
+	_underlined = underlined;
+#endif
+}
+
+- (bool)isBlinking
+{
+	return _blinking;
+}
+
+- (void)setBlinking: (bool)blinking
+{
+#ifndef OF_MSDOS
+	if (!self.hasTerminal)
+		return;
+
+	if (blinking == _blinking)
+		return;
+
+	[self writeString: (blinking ? @"\033[5m" : @"\033[25m")];
+
+	_blinking = blinking;
+#endif
+}
+
+- (bool)isCursorVisible
+{
+	return _cursorVisible;
+}
+
+- (void)setCursorVisible: (bool)visible
+{
+	if (!self.hasTerminal)
+		return;
+
+	if (visible == _cursorVisible)
+		return;
+
+#ifdef OF_MSDOS
+	_setcursortype(visible ? _NORMALCURSOR : _NOCURSOR);
+#else
+	[self writeString: (visible ? @"\033[?25h" : @"\033[?25l")];
+#endif
+
+	_cursorVisible = visible;
+}
+
 - (void)reset
 {
 	if (!self.hasTerminal)
@@ -663,6 +966,14 @@ colorToANSI(OFColor *color)
 #else
 	[self writeString: @"\033[0m"];
 #endif
+
+	objc_release(_foregroundColor);
+	objc_release(_backgroundColor);
+	_foregroundColor = _backgroundColor = nil;
+	_bold = false;
+	_italic = false;
+	_underlined = false;
+	_blinking = false;
 }
 
 - (void)clear
