@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -23,6 +23,7 @@
 #include <string.h>
 
 #import "OFString+PercentEncoding.h"
+#import "OFString+Private.h"
 #import "OFCharacterSet.h"
 
 #import "OFInvalidFormatException.h"
@@ -54,7 +55,7 @@ int _OFString_PercentEncoding_reference;
 			char buffer[4];
 			size_t bufferLen;
 
-			if ((bufferLen = OFUTF8StringEncode(c, buffer)) == 0)
+			if ((bufferLen = _OFUTF8StringEncode(c, buffer)) == 0)
 				@throw [OFInvalidEncodingException exception];
 
 			for (size_t j = 0; j < bufferLen; j++) {
