@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -24,6 +24,7 @@
 #include <errno.h>
 
 #import "OFZIPArchive.h"
+#import "OFZIPArchive+Private.h"
 #import "OFZIPArchiveEntry.h"
 #import "OFZIPArchiveEntry+Private.h"
 #import "OFArchiveIRIHandler.h"
@@ -121,7 +122,7 @@ OF_DIRECT_MEMBERS
 @end
 
 uint32_t
-OFZIPArchiveReadField32(const uint8_t **data, uint16_t *size)
+_OFZIPArchiveReadField32(const uint8_t **data, uint16_t *size)
 {
 	uint32_t field = 0;
 
@@ -138,7 +139,7 @@ OFZIPArchiveReadField32(const uint8_t **data, uint16_t *size)
 }
 
 uint64_t
-OFZIPArchiveReadField64(const uint8_t **data, uint16_t *size)
+_OFZIPArchiveReadField64(const uint8_t **data, uint16_t *size)
 {
 	uint64_t field = 0;
 
@@ -177,8 +178,8 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 			@throw [OFInvalidFormatException exception];
 
 		archive->_diskNumber = *diskNumber;
-		archive->_stream = [stream retain];
-		[oldStream release];
+		archive->_stream = objc_retain(stream);
+		objc_release(oldStream);
 	}
 
 	@try {
@@ -193,17 +194,19 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 
 + (instancetype)archiveWithStream: (OFStream *)stream mode: (OFString *)mode
 {
-	return [[[self alloc] initWithStream: stream mode: mode] autorelease];
+	return objc_autoreleaseReturnValue([[self alloc] initWithStream: stream
+								   mode: mode]);
 }
 
 + (instancetype)archiveWithIRI: (OFIRI *)IRI mode: (OFString *)mode
 {
-	return [[[self alloc] initWithIRI: IRI mode: mode] autorelease];
+	return objc_autoreleaseReturnValue([[self alloc] initWithIRI: IRI
+								mode: mode]);
 }
 
 + (OFIRI *)IRIForFilePath: (OFString *)path inArchiveWithIRI: (OFIRI *)IRI
 {
-	return OFArchiveIRIHandlerIRIForFileInArchive(@"zip", path, IRI);
+	return _OFArchiveIRIHandlerIRIForFileInArchive(@"zip", path, IRI);
 }
 
 - (instancetype)init
@@ -225,7 +228,7 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 		else
 			@throw [OFInvalidArgumentException exception];
 
-		_stream = [stream retain];
+		_stream = objc_retain(stream);
 		_entries = [[OFMutableArray alloc] init];
 		_pathToEntryMap = [[OFMutableDictionary alloc] init];
 
@@ -248,10 +251,10 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 		 * to write anything to it on error - after all, it might not
 		 * be a ZIP file which we would destroy otherwise.
 		 */
-		[_stream release];
+		objc_release(_stream);
 		_stream = nil;
 
-		[self release];
+		objc_release(self);
 		@throw e;
 	}
 
@@ -269,7 +272,7 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 		else
 			stream = [OFIRIHandler openItemAtIRI: IRI mode: mode];
 	} @catch (id e) {
-		[self release];
+		objc_release(self);
 		@throw e;
 	}
 
@@ -285,10 +288,10 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 	if (_stream != nil)
 		[self close];
 
-	[_stream release];
-	[_archiveComment release];
-	[_entries release];
-	[_pathToEntryMap release];
+	objc_release(_stream);
+	objc_release(_archiveComment);
+	objc_release(_entries);
+	objc_release(_pathToEntryMap);
 
 	[super dealloc];
 }
@@ -330,7 +333,7 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 	    _centralDirectoryEntries == 0xFFFF ||
 	    _centralDirectorySize == 0xFFFFFFFF ||
 	    _centralDirectoryOffset == 0xFFFFFFFF) {
-		uint32_t diskNumber;
+		uint32_t diskNumber, numDisks;
 		int64_t offset64;
 		uint64_t size;
 
@@ -347,7 +350,12 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 		 */
 		diskNumber = [_stream readLittleEndianInt32];
 		offset64 = [_stream readLittleEndianInt64];
-		_diskNumber = _lastDiskNumber = [_stream readLittleEndianInt32];
+		numDisks = [_stream readLittleEndianInt32];
+
+		if (numDisks == 0)
+			numDisks = 1;
+
+		_diskNumber = _lastDiskNumber = numDisks - 1;
 
 		if (offset64 < 0 || (OFStreamOffset)offset64 != offset64)
 			@throw [OFOutOfRangeException exception];
@@ -428,12 +436,12 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 				@throw [OFInvalidFormatException exception];
 
 			_diskNumber++;
-			_stream = [stream retain];
-			[oldStream release];
+			_stream = objc_retain(stream);
+			objc_release(oldStream);
 		}
 
-		entry = [[[OFZIPArchiveEntry alloc]
-		    of_initWithStream: _stream] autorelease];
+		entry = objc_autorelease(
+		    [[OFZIPArchiveEntry alloc] of_initWithStream: _stream]);
 
 		if ([_pathToEntryMap objectForKey: entry.fileName] != nil)
 			@throw [OFInvalidFormatException exception];
@@ -447,7 +455,7 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 
 - (OFArray *)entries
 {
-	return [[_entries copy] autorelease];
+	return objc_autoreleaseReturnValue([_entries copy]);
 }
 
 - (OFString *)archiveComment
@@ -465,7 +473,7 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 
 	old = _archiveComment;
 	_archiveComment = [comment copy];
-	[old release];
+	objc_release(old);
 
 	objc_autoreleasePoolPop(pool);
 }
@@ -503,8 +511,8 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 
 	seekOrThrowInvalidFormat(self, &startDiskNumber,
 	    (OFStreamOffset)offset64, OFSeekSet);
-	localFileHeader = [[[OFZIPArchiveLocalFileHeader alloc]
-	    initWithStream: _stream] autorelease];
+	localFileHeader = objc_autorelease(
+	    [[OFZIPArchiveLocalFileHeader alloc] initWithStream: _stream]);
 
 	if (![localFileHeader matchesEntry: entry])
 		@throw [OFInvalidFormatException exception];
@@ -520,12 +528,12 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 
 	objc_autoreleasePoolPop(pool);
 
-	_lastReturnedStream = [[[OFZIPArchiveFileReadStream alloc]
+	_lastReturnedStream = [[OFZIPArchiveFileReadStream alloc]
 	    of_initWithArchive: self
 			stream: _stream
-			 entry: entry] autorelease];
+			 entry: entry];
 
-	return _lastReturnedStream;
+	return objc_autoreleaseReturnValue(_lastReturnedStream);
 }
 
 - (OFStream *)streamForWritingEntry: (OFZIPArchiveEntry *)entry_
@@ -546,7 +554,7 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 		@throw [OFInvalidArgumentException exception];
 
 	pool = objc_autoreleasePoolPush();
-	entry = [[entry_ mutableCopy] autorelease];
+	entry = objc_autorelease([entry_ mutableCopy]);
 
 	if ([_pathToEntryMap objectForKey: entry.fileName] != nil)
 		@throw [OFOpenItemFailedException
@@ -601,7 +609,7 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 	[_stream writeLittleEndianInt16: extraFieldLength + 20];
 	offsetAdd += 4 + (5 * 2) + (3 * 4) + (2 * 2);
 
-	[_stream writeString: fileName];
+	[_stream writeString: fileName encoding: OFStringEncodingUTF8];
 	offsetAdd += fileNameLength;
 
 	[_stream writeLittleEndianInt16: OFZIPArchiveEntryExtraFieldTagZIP64];
@@ -631,7 +639,7 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 
 	objc_autoreleasePoolPop(pool);
 
-	return [_lastReturnedStream autorelease];
+	return objc_autoreleaseReturnValue(_lastReturnedStream);
 }
 
 - (void)of_writeCentralDirectory
@@ -676,9 +684,11 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 	[_stream writeLittleEndianInt16: 0xFFFF];	/* CD entries */
 	[_stream writeLittleEndianInt32: 0xFFFFFFFF];	/* CD size */
 	[_stream writeLittleEndianInt32: 0xFFFFFFFF];	/* CD offset */
-	[_stream writeLittleEndianInt16: _archiveComment.UTF8StringLength];
+	[_stream writeLittleEndianInt16: [_archiveComment
+	     cStringLengthWithEncoding: OFStringEncodingCodepage437]];
 	if (_archiveComment != nil)
-		[_stream writeString: _archiveComment];
+		[_stream writeString: _archiveComment
+			    encoding: OFStringEncodingCodepage437];
 
 	objc_autoreleasePoolPop(pool);
 }
@@ -698,7 +708,7 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 	if (_mode == modeWrite || _mode == modeAppend)
 		[self of_writeCentralDirectory];
 
-	[_stream release];
+	objc_release(_stream);
 	_stream = nil;
 }
 @end
@@ -735,8 +745,8 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 		_fileName = [[stream readStringWithLength: fileNameLength
 						 encoding: encoding] copy];
 		if (extraFieldLength > 0)
-			extraField = [[[stream readDataWithCount:
-			    extraFieldLength] mutableCopy] autorelease];
+			extraField = objc_autorelease([[stream
+			    readDataWithCount: extraFieldLength] mutableCopy]);
 
 		ZIP64Index = OFZIPArchiveEntryExtraFieldFind(extraField,
 		    OFZIPArchiveEntryExtraFieldTagZIP64, &ZIP64Size);
@@ -747,12 +757,10 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 			OFRange range =
 			    OFMakeRange(ZIP64Index - 4, ZIP64Size + 4);
 
-			if (_uncompressedSize == 0xFFFFFFFF)
-				_uncompressedSize = OFZIPArchiveReadField64(
-				    &ZIP64, &ZIP64Size);
-			if (_compressedSize == 0xFFFFFFFF)
-				_compressedSize = OFZIPArchiveReadField64(
-				    &ZIP64, &ZIP64Size);
+			_uncompressedSize = _OFZIPArchiveReadField64(
+			    &ZIP64, &ZIP64Size);
+			_compressedSize = _OFZIPArchiveReadField64(
+			    &ZIP64, &ZIP64Size);
 
 			if (ZIP64Size > 0)
 				@throw [OFInvalidFormatException exception];
@@ -767,7 +775,7 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 
 		objc_autoreleasePoolPop(pool);
 	} @catch (id e) {
-		[self release];
+		objc_release(self);
 		@throw e;
 	}
 
@@ -776,17 +784,15 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 
 - (void)dealloc
 {
-	[_fileName release];
-	[_extraField release];
+	objc_release(_fileName);
+	objc_release(_extraField);
 
 	[super dealloc];
 }
 
 - (bool)matchesEntry: (OFZIPArchiveEntry *)entry
 {
-	if (_compressionMethod != entry.compressionMethod ||
-	    _lastModifiedFileTime != entry.of_lastModifiedFileTime ||
-	    _lastModifiedFileDate != entry.of_lastModifiedFileDate)
+	if (_compressionMethod != entry.compressionMethod)
 		return false;
 
 	if (!(_generalPurposeBitFlag & (1u << 3)))
@@ -810,12 +816,12 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 	self = [super init];
 
 	@try {
-		_archive = [archive retain];
+		_archive = objc_retain(archive);
 		_compressionMethod = entry.compressionMethod;
 
 		switch (_compressionMethod) {
 		case OFZIPArchiveEntryCompressionMethodNone:
-			_decompressedStream = [_archive->_stream retain];
+			_decompressedStream = objc_retain(_archive->_stream);
 			break;
 		case OFZIPArchiveEntryCompressionMethodDeflate:
 			_decompressedStream = [[OFInflateStream alloc]
@@ -835,7 +841,7 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 		_toRead = entry.uncompressedSize;
 		_CRC32 = ~0;
 	} @catch (id e) {
-		[self release];
+		objc_release(self);
 		@throw e;
 	}
 
@@ -847,12 +853,12 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 	if (_decompressedStream != nil)
 		[self close];
 
-	[_entry release];
+	objc_release(_entry);
 
 	if (_archive->_lastReturnedStream == self)
 		_archive->_lastReturnedStream = nil;
 
-	[_archive release];
+	objc_release(_archive);
 
 	[super dealloc];
 }
@@ -892,14 +898,14 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 			@throw [OFInvalidFormatException exception];
 
 		_archive->_diskNumber++;
-		_archive->_stream = [stream retain];
-		[oldStream release];
+		_archive->_stream = objc_retain(stream);
+		objc_release(oldStream);
 
 		switch (_compressionMethod) {
 		case OFZIPArchiveEntryCompressionMethodNone:
 			oldDecompressedStream = _decompressedStream;
-			_decompressedStream = [_archive->_stream retain];
-			[oldDecompressedStream release];
+			_decompressedStream = objc_retain(_archive->_stream);
+			objc_release(oldDecompressedStream);
 			break;
 		case OFZIPArchiveEntryCompressionMethodDeflate:
 		case OFZIPArchiveEntryCompressionMethodDeflate64:
@@ -924,7 +930,7 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 	ret = [_decompressedStream readIntoBuffer: buffer length: length];
 
 	_toRead -= ret;
-	_CRC32 = OFCRC32(_CRC32, buffer, ret);
+	_CRC32 = _OFCRC32(_CRC32, buffer, ret);
 
 	if (_toRead == 0) {
 		_atEndOfStream = true;
@@ -960,7 +966,7 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 	if (_decompressedStream == nil)
 		@throw [OFNotOpenException exceptionWithObject: self];
 
-	[_decompressedStream release];
+	objc_release(_decompressedStream);
 	_decompressedStream = nil;
 
 	[super close];
@@ -976,9 +982,9 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 {
 	self = [super init];
 
-	_archive = [archive retain];
-	_stream = [stream retain];
-	_entry = [entry retain];
+	_archive = objc_retain(archive);
+	_stream = objc_retain(stream);
+	_entry = objc_retain(entry);
 	_CRC32 = ~0;
 	_CRC32Offset = CRC32Offset;
 	_size64Offset = size64Offset;
@@ -991,12 +997,12 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 	if (_stream != nil)
 		[self close];
 
-	[_entry release];
+	objc_release(_entry);
 
 	if (_archive->_lastReturnedStream == self)
 		_archive->_lastReturnedStream = nil;
 
-	[_archive release];
+	objc_release(_archive);
 
 	[super dealloc];
 }
@@ -1017,7 +1023,7 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 		OFEnsure(e.bytesWritten <= length);
 
 		_bytesWritten += (unsigned long long)e.bytesWritten;
-		_CRC32 = OFCRC32(_CRC32, buffer, e.bytesWritten);
+		_CRC32 = _OFCRC32(_CRC32, buffer, e.bytesWritten);
 
 		if (e.errNo == EWOULDBLOCK || e.errNo == EAGAIN)
 			return e.bytesWritten;
@@ -1026,7 +1032,7 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 	}
 
 	_bytesWritten += (unsigned long long)length;
-	_CRC32 = OFCRC32(_CRC32, buffer, length);
+	_CRC32 = _OFCRC32(_CRC32, buffer, length);
 
 	return length;
 }
@@ -1061,7 +1067,7 @@ seekOrThrowInvalidFormat(OFZIPArchive *archive, const uint32_t *diskNumber,
 		[_stream writeLittleEndianInt64: (uint64_t)_bytesWritten];
 	}
 
-	[_stream release];
+	objc_release(_stream);
 	_stream = nil;
 
 	_entry.CRC32 = ~_CRC32;
