@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -19,7 +19,7 @@
 
 #import <Foundation/NSDictionary.h>
 
-#import "NSBridging.h"
+#import "OFNSToOFBridging.h"
 
 OF_ASSUME_NONNULL_BEGIN
 
@@ -28,18 +28,18 @@ OF_ASSUME_NONNULL_BEGIN
 #ifdef __cplusplus
 extern "C" {
 #endif
-extern int _NSDictionary_OFObject_reference OF_VISIBILITY_HIDDEN;
+extern int _NSDictionary_OFObject_reference OF_VISIBILITY_INTERNAL;
 #ifdef __cplusplus
 }
 #endif
 
 /**
- * @category NSDictionary (OFObject) \
- *	     NSDictionary+OFObject.h ObjFWBridge/NSDictionary+OFObject.h
+ * @category NSDictionary (OFObject)
+ *	     NSDictionary+OFObject.h ObjFWBridge/ObjFWBridge.h
  *
- * @brief Support for bridging NSDictionaries to OFDictionaries.
+ * @brief Support for bridging an NSDictionary to an @ref OFDictionary.
  */
-@interface NSDictionary (OFObject) <NSBridging>
+@interface NSDictionary (OFObject) <OFNSToOFBridging>
 @property (readonly, nonatomic) OFDictionary *OFObject;
 @end
 
