@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -19,7 +19,7 @@
 
 #import <Foundation/NSEnumerator.h>
 
-#import "NSBridging.h"
+#import "OFNSToOFBridging.h"
 
 OF_ASSUME_NONNULL_BEGIN
 
@@ -28,18 +28,18 @@ OF_ASSUME_NONNULL_BEGIN
 #ifdef __cplusplus
 extern "C" {
 #endif
-extern int _NSEnumerator_OFObject_reference OF_VISIBILITY_HIDDEN;
+extern int _NSEnumerator_OFObject_reference OF_VISIBILITY_INTERNAL;
 #ifdef __cplusplus
 }
 #endif
 
 /**
- * @category NSEnumerator (OFObject) \
- *	     NSEnumerator+OFObject.h ObjFWBridge/NSEnumerator+OFObject.h
+ * @category NSEnumerator (OFObject)
+ *	     NSEnumerator+OFObject.h ObjFWBridge/ObjFWBridge.h
  *
- * @brief Support for bridging NSEnumerators to OFEnumerators.
+ * @brief Support for bridging an NSEnumerator to an @ref OFEnumerator.
  */
-@interface NSEnumerator (OFObject) <NSBridging>
+@interface NSEnumerator (OFObject) <OFNSToOFBridging>
 @property (readonly, nonatomic) OFEnumerator *OFObject;
 @end
 
