@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -27,24 +27,24 @@
 # endif
 #endif
 
-#import "OFBridging.h"
+#import "OFOFToNSBridging.h"
 
 OF_ASSUME_NONNULL_BEGIN
 
 #ifdef __cplusplus
 extern "C" {
 #endif
-extern int _OFArray_NSObject_reference OF_VISIBILITY_HIDDEN;
+extern int _OFArray_NSObject_reference OF_VISIBILITY_INTERNAL;
 #ifdef __cplusplus
 }
 #endif
 
 /**
- * @category OFArray (NSObject) \
- *	     OFArray+NSObject.h ObjFWBridge/OFArray+NSObject.h
- * @brief Support for bridging OFArrays to NSArrays.
+ * @category OFArray (NSObject) OFArray+NSObject.h ObjFWBridge/ObjFWBridge.h
+ *
+ * @brief Support for bridging an @ref OFArray to an NSArray.
  */
-@interface OFArray (NSObject) <OFBridging>
+@interface OFArray (NSObject) <OFOFToNSBridging>
 @property (readonly, nonatomic) NSArray *NSObject;
 @end
 
