@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -26,6 +26,7 @@ int _OFArray_NSObject_reference;
 @implementation OFArray (NSObject)
 - (NSArray *)NSObject
 {
-	return [[[NSOFArray alloc] initWithOFArray: self] autorelease];
+	return objc_autoreleaseReturnValue(
+	    [[NSOFArray alloc] initWithOFArray: self]);
 }
 @end
