@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -18,6 +18,8 @@
  */
 
 #import "NSArray+OFObject.h"
+#import "NSData+OFObject.h"
+#import "NSDate+OFObject.h"
 #import "NSDictionary+OFObject.h"
 #import "NSEnumerator+OFObject.h"
 #import "NSNumber+OFObject.h"
@@ -25,9 +27,11 @@
 #import "NSString+OFObject.h"
 
 #import "OFArray+NSObject.h"
-#import "OFException+Swift.h"
+#import "OFData+NSObject.h"
+#import "OFDate+NSObject.h"
 #import "OFDictionary+NSObject.h"
 #import "OFEnumerator+NSObject.h"
+#import "OFException+OFSwift.h"
 #import "OFNumber+NSObject.h"
 #import "OFSet+NSObject.h"
 #import "OFString+NSObject.h"
