@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -31,9 +31,10 @@
 		    addressFamily: (OFSocketAddressFamily)addressFamily
 			errorCode: (OFDNSResolverErrorCode)errorCode
 {
-	return [[[self alloc] initWithHost: host
-			     addressFamily: addressFamily
-				 errorCode: errorCode] autorelease];
+	return objc_autoreleaseReturnValue(
+	    [[self alloc] initWithHost: host
+			 addressFamily: addressFamily
+			     errorCode: errorCode]);
 }
 
 + (instancetype)exception
@@ -52,7 +53,7 @@
 		_addressFamily = addressFamily;
 		_errorCode = errorCode;
 	} @catch (id e) {
-		[self release];
+		objc_release(self);
 		@throw e;
 	}
 
@@ -66,7 +67,7 @@
 
 - (void)dealloc
 {
-	[_host release];
+	objc_release(_host);
 
 	[super dealloc];
 }
@@ -75,6 +76,6 @@
 {
 	return [OFString stringWithFormat:
 	    @"The host %@ could not be resolved: %@",
-	    _host, OFDNSResolverErrorCodeDescription(_errorCode)];
+	    _host, _OFDNSResolverErrorCodeDescription(_errorCode)];
 }
 @end
