@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -22,6 +22,9 @@
 #import "OHDualSenseGamepad.h"
 #import "OHDualSenseGamepad+Private.h"
 #import "OFDictionary.h"
+#ifdef OF_HAVE_GCF
+# import "OFString+NSObject.h"
+#endif
 #import "OHEmulatedGameControllerTriggerButton.h"
 #import "OHGameControllerAxis.h"
 #import "OHGameControllerButton.h"
@@ -41,9 +44,49 @@ static OFString *const buttonNames[] = {
 };
 static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 
+#ifdef OF_HAVE_GCF
+static OFDictionary<OFString *, NSString *> *buttonsMap;
+static OFDictionary<OFString *, NSString *> *directionalPadsMap;
+#endif
+
 @implementation OHDualSenseGamepad
 @synthesize buttons = _buttons, directionalPads = _directionalPads;
 
+#ifdef OF_HAVE_GCF
++ (void)initialize
+{
+	void *pool;
+
+	if (self != [OHDualSenseGamepad class])
+		return;
+
+	pool = objc_autoreleasePoolPush();
+
+	buttonsMap = [[OFDictionary alloc] initWithKeysAndObjects:
+	    @"Triangle", @"Button Y".NSObject,
+	    @"Cross", @"Button A".NSObject,
+	    @"Square", @"Button X".NSObject,
+	    @"Circle", @"Button B".NSObject,
+	    @"L1", @"Left Shoulder".NSObject,
+	    @"R1", @"Right Shoulder".NSObject,
+	    @"L2", @"Left Trigger".NSObject,
+	    @"R2", @"Right Trigger".NSObject,
+	    @"L3", @"Left Thumbstick".NSObject,
+	    @"R3", @"Right Thumbstick".NSObject,
+	    @"Options", @"Button Menu".NSObject,
+	    @"Create", @"Button Options".NSObject,
+	    @"PS", @"Button Home".NSObject,
+	    nil];
+	directionalPadsMap = [[OFDictionary alloc] initWithKeysAndObjects:
+	    @"Left Thumbstick", @"Left Thumbstick".NSObject,
+	    @"Right Thumbstick", @"Right Thumbstick".NSObject,
+	    @"D-Pad", @"Direction Pad".NSObject,
+	    nil];
+
+	objc_autoreleasePoolPop(pool);
+}
+#endif
+
 - (instancetype)init
 {
 	OF_INVALID_INIT_METHOD
@@ -59,7 +102,10 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 		    [OFMutableDictionary dictionaryWithCapacity: numButtons];
 		OHGameControllerButton *button;
 		OFMutableDictionary *directionalPads;
-		OHGameControllerAxis *axis, *xAxis, *yAxis;
+#if defined(OF_LINUX) && defined(OF_HAVE_FILES)
+		OHGameControllerAxis *axis;
+#endif
+		OHGameControllerAxis *xAxis, *yAxis;
 		OHGameControllerDirectionalPad *directionalPad;
 
 		for (size_t i = 0; i < numButtons; i++) {
@@ -69,6 +115,7 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 			[buttons setObject: button forKey: buttonNames[i]];
 		}
 
+#if defined(OF_LINUX) && defined(OF_HAVE_FILES)
 		axis = [OHGameControllerAxis oh_elementWithName: @"L2"
 							 analog: true];
 		button = [OHEmulatedGameControllerTriggerButton
@@ -82,36 +129,48 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 		    oh_buttonWithName: @"R2"
 				 axis: axis];
 		[buttons setObject: button forKey: @"R2"];
+#else
+		button = [OHGameControllerButton oh_elementWithName: @"L2"
+							     analog: true];
+		[buttons setObject: button forKey: @"L2"];
+		button = [OHGameControllerButton oh_elementWithName: @"R2"
+							     analog: true];
+		[buttons setObject: button forKey: @"R2"];
+#endif
 
 		[buttons makeImmutable];
-		_buttons = [buttons retain];
+		_buttons = [buttons copy];
 
 		directionalPads =
 		    [OFMutableDictionary dictionaryWithCapacity: 3];
 
-		xAxis = [OHGameControllerAxis oh_elementWithName: @"X"
-							  analog: true];
-		yAxis = [OHGameControllerAxis oh_elementWithName: @"Y"
-							  analog: true];
+		xAxis = [OHGameControllerAxis
+		    oh_elementWithName: @"Left Thumbstick X"
+				analog: true];
+		yAxis = [OHGameControllerAxis
+		    oh_elementWithName: @"Left Thumbstick Y"
+				analog: true];
 		directionalPad = [OHGameControllerDirectionalPad
-		    oh_padWithName: @"Left Stick"
+		    oh_padWithName: @"Left Thumbstick"
 			     xAxis: xAxis
 			     yAxis: yAxis
 			    analog: true];
 		[directionalPads setObject: directionalPad
-				    forKey: @"Left Stick"];
+				    forKey: @"Left Thumbstick"];
 
-		xAxis = [OHGameControllerAxis oh_elementWithName: @"RX"
-							  analog: true];
-		yAxis = [OHGameControllerAxis oh_elementWithName: @"RY"
-							  analog: true];
+		xAxis = [OHGameControllerAxis
+		    oh_elementWithName: @"Right Thumbstick X"
+				analog: true];
+		yAxis = [OHGameControllerAxis
+		    oh_elementWithName: @"Right Thumbstick Y"
+				analog: true];
 		directionalPad = [OHGameControllerDirectionalPad
-		    oh_padWithName: @"Right Stick"
+		    oh_padWithName: @"Right Thumbstick"
 			     xAxis: xAxis
 			     yAxis: yAxis
 			    analog: true];
 		[directionalPads setObject: directionalPad
-				    forKey: @"Right Stick"];
+				    forKey: @"Right Thumbstick"];
 
 		xAxis = [OHGameControllerAxis oh_elementWithName: @"D-Pad X"
 							  analog: false];
@@ -125,11 +184,11 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 		[directionalPads setObject: directionalPad forKey: @"D-Pad"];
 
 		[directionalPads makeImmutable];
-		_directionalPads = [directionalPads retain];
+		_directionalPads = [directionalPads copy];
 
 		objc_autoreleasePoolPop(pool);
 	} @catch (id e) {
-		[self release];
+		objc_release(self);
 		@throw e;
 	}
 
@@ -138,8 +197,8 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 
 - (void)dealloc
 {
-	[_buttons release];
-	[_directionalPads release];
+	objc_release(_buttons);
+	objc_release(_directionalPads);
 
 	[super dealloc];
 }
@@ -216,12 +275,12 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 
 - (OHGameControllerDirectionalPad *)leftThumbstick
 {
-	return [_directionalPads objectForKey: @"Left Stick"];
+	return [_directionalPads objectForKey: @"Left Thumbstick"];
 }
 
 - (OHGameControllerDirectionalPad *)rightThumbstick
 {
-	return [_directionalPads objectForKey: @"Right Stick"];
+	return [_directionalPads objectForKey: @"Right Thumbstick"];
 }
 
 - (OHGameControllerDirectionalPad *)dPad
@@ -279,13 +338,17 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 {
 	switch (axis) {
 	case ABS_X:
-		return [[_directionalPads objectForKey: @"Left Stick"] xAxis];
+		return [[_directionalPads objectForKey: @"Left Thumbstick"]
+		    xAxis];
 	case ABS_Y:
-		return [[_directionalPads objectForKey: @"Left Stick"] yAxis];
+		return [[_directionalPads objectForKey: @"Left Thumbstick"]
+		    yAxis];
 	case ABS_RX:
-		return [[_directionalPads objectForKey: @"Right Stick"] xAxis];
+		return [[_directionalPads objectForKey: @"Right Thumbstick"]
+		    xAxis];
 	case ABS_RY:
-		return [[_directionalPads objectForKey: @"Right Stick"] yAxis];
+		return [[_directionalPads objectForKey: @"Right Thumbstick"]
+		    yAxis];
 	case ABS_HAT0X:
 		return [[_directionalPads objectForKey: @"D-Pad"] xAxis];
 	case ABS_HAT0Y:
@@ -299,4 +362,21 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 	}
 }
 #endif
+
+#ifdef OF_HAVE_GCF
+- (OFDictionary<OFString *, NSString *> *)oh_buttonsMap
+{
+	return buttonsMap;
+}
+
+- (OFDictionary<OFString *, NSString *> *)oh_axesMap
+{
+	return [OFDictionary dictionary];
+}
+
+- (OFDictionary<OFString *, NSString *> *)oh_directionalPadsMap
+{
+	return directionalPadsMap;
+}
+#endif
 @end
