@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -20,17 +20,24 @@
 #include "config.h"
 
 #import "OHGameControllerElement.h"
+#import "OHGameControllerElement+Private.h"
 
 @implementation OHGameControllerElement
 @synthesize name = _name, analog = _analog;
 
++ (instancetype)oh_elementWithName: (OFString *)name analog: (bool)analog
+{
+	return objc_autoreleaseReturnValue(
+	    [[self alloc] oh_initWithName: name
+				   analog: analog]);
+}
+
 - (instancetype)init
 {
 	OF_INVALID_INIT_METHOD
 }
 
-- (instancetype)initWithName: (OFString *)name
-		      analog: (bool)analog
+- (instancetype)oh_initWithName: (OFString *)name analog: (bool)analog
 {
 	self = [super init];
 
@@ -38,7 +45,7 @@
 		_name = [name copy];
 		_analog = analog;
 	} @catch (id e) {
-		[self release];
+		objc_release(self);
 		@throw e;
 	}
 
@@ -47,7 +54,7 @@
 
 - (void)dealloc
 {
-	[_name release];
+	objc_release(_name);
 
 	[super dealloc];
 }
