@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -42,9 +42,9 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 	@try {
 		void *pool = objc_autoreleasePoolPush();
 		OFMutableDictionary *buttons =
-		    [[_buttons mutableCopy] autorelease];
+		    objc_autorelease([_buttons mutableCopy]);
 		OFMutableDictionary *directionalPads =
-		    [[_directionalPads mutableCopy] autorelease];
+		    objc_autorelease([_directionalPads mutableCopy]);
 		OHGameControllerAxis *xAxis, *yAxis;
 		OHGameControllerDirectionalPad *directionalPad;
 
@@ -56,10 +56,12 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 			[buttons setObject: button forKey: buttonNames[i]];
 		}
 
-		xAxis = [OHGameControllerAxis oh_elementWithName: @"X"
-							  analog: true];
-		yAxis = [OHGameControllerAxis oh_elementWithName: @"Y"
-							  analog: true];
+		xAxis = [OHGameControllerAxis
+		    oh_elementWithName: @"Analog Stick X"
+				analog: true];
+		yAxis = [OHGameControllerAxis
+		    oh_elementWithName: @"Analog Stick Y"
+				analog: true];
 		directionalPad = [OHGameControllerDirectionalPad
 		    oh_padWithName: @"Analog Stick"
 			     xAxis: xAxis
@@ -69,16 +71,18 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 				    forKey: @"Analog Stick"];
 
 		[buttons makeImmutable];
-		[_buttons release];
-		_buttons = [buttons retain];
+		objc_release(_buttons);
+		_buttons = nil;
+		_buttons = [buttons copy];
 
 		[directionalPads makeImmutable];
-		[_directionalPads release];
-		_directionalPads = [directionalPads retain];
+		objc_release(_directionalPads);
+		_directionalPads = nil;
+		_directionalPads = [directionalPads copy];
 
 		objc_autoreleasePoolPop(pool);
 	} @catch (id e) {
-		[self release];
+		objc_release(self);
 		@throw e;
 	}
 
