@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -101,9 +101,12 @@
 # define OF_WEAK_REF(sym) __attribute__((__weakref__(sym)))
 # if defined(OF_ELF) || defined(OF_MACHO)
 #  define OF_VISIBILITY_HIDDEN __attribute__((__visibility__("hidden")))
+#  define OF_VISIBILITY_INTERNAL __attribute__((__visibility__("internal")))
 # else
 #  define OF_VISIBILITY_HIDDEN
+#  define OF_VISIBILITY_INTERNAL
 # endif
+# define OF_MALLOC_FUNC __attribute__((__malloc__))
 #else
 # define OF_INLINE inline
 # define OF_LIKELY(cond) (cond)
@@ -112,6 +115,8 @@
 # define OF_NO_RETURN_FUNC
 # define OF_WEAK_REF(sym)
 # define OF_VISIBILITY_HIDDEN
+# define OF_VISIBILITY_INTERNAL
+# define OF_MALLOC_FUNC
 #endif
 
 #if __STDC_VERSION__ >= 201112L
@@ -141,6 +146,8 @@
 
 #define OF_PREPROCESSOR_CONCAT2(a, b) a##b
 #define OF_PREPROCESSOR_CONCAT(a, b) OF_PREPROCESSOR_CONCAT2(a, b)
+#define OF_PREPROCESSOR_STRINGIFY2(x) #x
+#define OF_PREPROCESSOR_STRINGIFY(x) OF_PREPROCESSOR_STRINGIFY2(x)
 
 #if __OBJFW_RUNTIME_ABI__ || (defined(OF_APPLE_RUNTIME) && defined(__OBJC2__))
 # define OF_HAVE_NONFRAGILE_IVARS
@@ -333,14 +340,23 @@
 # define OF_SWIFT_NAME(name)
 #endif
 
-#if __has_attribute(__objc_direct__) && defined(OF_APPLE_RUNTIME)
-# define OF_DIRECT __attribute__((__objc_direct__))
-#else
+#if defined(OF_APPLE_RUNTIME) || (defined(OF_OBJFW_RUNTIME) && \
+    defined(__clang_major__) && __clang_major__ >= 21)
+# if __has_attribute(__objc_direct__)
+#  define OF_DIRECT __attribute__((__objc_direct__))
+#  define OF_DIRECT_PROPERTY(...) (__VA_ARGS__, direct)
+# endif
+# if __has_attribute(__objc_direct_members__)
+#  define OF_DIRECT_MEMBERS __attribute__((__objc_direct_members__))
+# endif
+#endif
+#ifndef OF_DIRECT
 # define OF_DIRECT
 #endif
-#if __has_attribute(__objc_direct_members__) && defined(OF_APPLE_RUNTIME)
-# define OF_DIRECT_MEMBERS __attribute__((__objc_direct_members__))
-#else
+#ifndef OF_DIRECT_PROPERTY
+# define OF_DIRECT_PROPERTY
+#endif
+#ifndef OF_DIRECT_MEMBERS
 # define OF_DIRECT_MEMBERS
 #endif
 
@@ -423,7 +439,7 @@ extern void OFLog(OFConstantString *_Nonnull, ...);
 	@try {						\
 		OFMethodNotFound(self, _cmd);		\
 	} @catch (id e) {				\
-		[self release];				\
+		objc_release(self);			\
 		@throw e;				\
 	}						\
 							\
