@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -154,7 +154,7 @@ OFPlainThreadNew(OFPlainThread *thread, const char *name, void (*function)(id),
 		free(*thread);
 		@throw e;
 	} @finally {
-		[tags release];
+		objc_release(tags);
 	}
 
 	return 0;
