@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -52,6 +52,7 @@ extern char **environ;
 # define environ (*_NSGetEnviron())
 #endif
 
+OF_DIRECT_MEMBERS
 @interface OFSubprocess ()
 - (void)of_getArgv: (char ***)argv
     forProgramName: (OFString *)programName
@@ -62,23 +63,26 @@ extern char **environ;
 @implementation OFSubprocess
 + (instancetype)subprocessWithProgram: (OFString *)program
 {
-	return [[[self alloc] initWithProgram: program] autorelease];
+	return objc_autoreleaseReturnValue(
+	    [[self alloc] initWithProgram: program]);
 }
 
 + (instancetype)subprocessWithProgram: (OFString *)program
 			    arguments: (OFArray *)arguments
 {
-	return [[[self alloc] initWithProgram: program
-				    arguments: arguments] autorelease];
+	return objc_autoreleaseReturnValue(
+	    [[self alloc] initWithProgram: program
+				arguments: arguments]);
 }
 
 + (instancetype)subprocessWithProgram: (OFString *)program
 			  programName: (OFString *)programName
 			    arguments: (OFArray *)arguments
 {
-	return [[[self alloc] initWithProgram: program
-				  programName: programName
-				    arguments: arguments] autorelease];
+	return objc_autoreleaseReturnValue(
+	    [[self alloc] initWithProgram: program
+			      programName: programName
+				arguments: arguments]);
 }
 
 + (instancetype)subprocessWithProgram: (OFString *)program
@@ -86,10 +90,11 @@ extern char **environ;
 			    arguments: (OFArray *)arguments
 			  environment: (OFDictionary *)environment
 {
-	return [[[self alloc] initWithProgram: program
-				  programName: programName
-				    arguments: arguments
-				  environment: environment] autorelease];
+	return objc_autoreleaseReturnValue(
+	    [[self alloc] initWithProgram: program
+			      programName: programName
+				arguments: arguments
+			      environment: environment]);
 }
 
 - (instancetype)init
@@ -226,7 +231,7 @@ extern char **environ;
 
 		objc_autoreleasePoolPop(pool);
 	} @catch (id e) {
-		[self release];
+		objc_release(self);
 		@throw e;
 	}
 
