@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -27,61 +27,76 @@
 # import "OFMapTable.h"
 #endif
 
+#import "pre_ivar.h"
+
+#ifdef OF_HAVE_ATOMIC_OPS
+# import "OFAtomic.h"
+#endif
+
 struct Association {
 	id object;
 	objc_associationPolicy policy;
 };
 
 #ifdef OF_OBJFW_RUNTIME
-typedef struct objc_hashtable objc_hashtable;
+typedef struct objc_hashtable _objc_hashtable;
+
+/* Inlined and unncessary checks dropped for performance. */
+static OF_INLINE Class
+_object_getClass_fast(id object_)
+{
+	struct objc_object *object = (struct objc_object *)object_;
+
+	return object->isa;
+}
 #else
-typedef OFMapTable objc_hashtable;
+typedef OFMapTable _objc_hashtable;
 static const OFMapTableFunctions defaultFunctions = { NULL };
 
-static objc_hashtable *
-objc_hashtable_new(uint32_t (*hash)(const void *key),
+static OF_INLINE _objc_hashtable *
+_objc_hashtable_new(uint32_t (*hash)(const void *key),
     bool (*equal)(const void *key1, const void *key2), uint32_t size)
 {
 	return [[OFMapTable alloc] initWithKeyFunctions: defaultFunctions
 					objectFunctions: defaultFunctions];
 }
 
-static void
-objc_hashtable_set(objc_hashtable *hashtable, const void *key,
+static OF_INLINE void
+_objc_hashtable_set(_objc_hashtable *hashtable, const void *key,
     const void *object)
 {
 	return [hashtable setObject: (void *)object forKey: (void *)key];
 }
 
-static void *
-objc_hashtable_get(objc_hashtable *hashtable, const void *key)
+static OF_INLINE void *
+_objc_hashtable_get(_objc_hashtable *hashtable, const void *key)
 {
 	return [hashtable objectForKey: (void *)key];
 }
 
-static void
-objc_hashtable_delete(objc_hashtable *hashtable, const void *key)
+static OF_INLINE void
+_objc_hashtable_delete(_objc_hashtable *hashtable, const void *key)
 {
 	[hashtable removeObjectForKey: (void *)key];
 }
 
-static void
-objc_hashtable_free(objc_hashtable *hashtable)
+static OF_INLINE void
+_objc_hashtable_free(_objc_hashtable *hashtable)
 {
-	[hashtable release];
+	objc_release(hashtable);
 }
 
-# define OBJC_ERROR(...) abort()
+# define _OBJC_ERROR(...) abort()
 #endif
 
 #ifdef OF_HAVE_THREADS
-# define numSlots 8	/* needs to be a power of 2 */
+# define numSlots 16	/* needs to be a power of 2 */
 # import "OFPlainMutex.h"
 static OFSpinlock spinlocks[numSlots];
 #else
 # define numSlots 1
 #endif
-static objc_hashtable *hashtables[numSlots];
+static _objc_hashtable *hashtables[numSlots];
 
 static OF_INLINE size_t
 slotForObject(id object)
@@ -104,10 +119,10 @@ equal(const void *object1, const void *object2)
 OF_CONSTRUCTOR()
 {
 	for (size_t i = 0; i < numSlots; i++) {
-		hashtables[i] = objc_hashtable_new(hash, equal, 2);
+		hashtables[i] = _objc_hashtable_new(hash, equal, 2);
 #ifdef OF_HAVE_THREADS
 		if (OFSpinlockNew(&spinlocks[i]) != 0)
-			OBJC_ERROR("Failed to create spinlocks!");
+			_OBJC_ERROR("Failed to create spinlocks!");
 #endif
 	}
 }
@@ -123,7 +138,7 @@ objc_setAssociatedObject(id object, const void *key, id value,
 		break;
 	case OBJC_ASSOCIATION_RETAIN:
 	case OBJC_ASSOCIATION_RETAIN_NONATOMIC:
-		value = [value retain];
+		value = objc_retain(value);
 		break;
 	case OBJC_ASSOCIATION_COPY:
 	case OBJC_ASSOCIATION_COPY_NONATOMIC:
@@ -134,32 +149,39 @@ objc_setAssociatedObject(id object, const void *key, id value,
 		return;
 	}
 
+#if defined(OF_HAVE_ATOMIC_OPS) && defined(OF_OBJFW_RUNTIME)
+	if (!object_isTaggedPointer(object) &&
+	    (_object_getClass_fast(object)->info & _OBJC_CLASS_INFO_RUNTIME_RR))
+		OFAtomicIntOr(&_OBJC_PRE_IVARS(object)->info,
+		    _OBJC_OBJECT_INFO_ASSOCIATIONS);
+#endif
+
 	slot = slotForObject(object);
 
 #ifdef OF_HAVE_THREADS
 	if (OFSpinlockLock(&spinlocks[slot]) != 0)
-		OBJC_ERROR("Failed to lock spinlock!");
+		_OBJC_ERROR("Failed to lock spinlock!");
 
 	@try {
 #endif
-		objc_hashtable *objectHashtable;
+		_objc_hashtable *objectHashtable;
 		struct Association *association;
 
-		objectHashtable = objc_hashtable_get(hashtables[slot], object);
+		objectHashtable = _objc_hashtable_get(hashtables[slot], object);
 		if (objectHashtable == NULL) {
-			objectHashtable = objc_hashtable_new(hash, equal, 2);
-			objc_hashtable_set(hashtables[slot], object,
+			objectHashtable = _objc_hashtable_new(hash, equal, 2);
+			_objc_hashtable_set(hashtables[slot], object,
 			    objectHashtable);
 		}
 
-		association = objc_hashtable_get(objectHashtable, key);
+		association = _objc_hashtable_get(objectHashtable, key);
 		if (association != NULL) {
 			switch (association->policy) {
 			case OBJC_ASSOCIATION_RETAIN:
 			case OBJC_ASSOCIATION_RETAIN_NONATOMIC:
 			case OBJC_ASSOCIATION_COPY:
 			case OBJC_ASSOCIATION_COPY_NONATOMIC:
-				[association->object release];
+				objc_release(association->object);
 				break;
 			default:
 				break;
@@ -167,9 +189,9 @@ objc_setAssociatedObject(id object, const void *key, id value,
 		} else {
 			association = malloc(sizeof(*association));
 			if (association == NULL)
-				OBJC_ERROR("Failed to allocate association!");
+				_OBJC_ERROR("Failed to allocate association!");
 
-			objc_hashtable_set(objectHashtable, key, association);
+			_objc_hashtable_set(objectHashtable, key, association);
 		}
 
 		association->policy = policy;
@@ -177,7 +199,7 @@ objc_setAssociatedObject(id object, const void *key, id value,
 #ifdef OF_HAVE_THREADS
 	} @finally {
 		if (OFSpinlockUnlock(&spinlocks[slot]) != 0)
-			OBJC_ERROR("Failed to unlock spinlock!");
+			_OBJC_ERROR("Failed to unlock spinlock!");
 	}
 #endif
 }
@@ -190,25 +212,26 @@ objc_getAssociatedObject(id object, const void *key)
 
 #ifdef OF_HAVE_THREADS
 	if (OFSpinlockLock(&spinlocks[slot]) != 0)
-		OBJC_ERROR("Failed to lock spinlock!");
+		_OBJC_ERROR("Failed to lock spinlock!");
 
 	@try {
 #endif
-		objc_hashtable *objectHashtable;
+		_objc_hashtable *objectHashtable;
 		struct Association *association;
 
-		objectHashtable = objc_hashtable_get(hashtables[slot], object);
+		objectHashtable = _objc_hashtable_get(hashtables[slot], object);
 		if (objectHashtable == NULL)
 			return nil;
 
-		association = objc_hashtable_get(objectHashtable, key);
+		association = _objc_hashtable_get(objectHashtable, key);
 		if (association == NULL)
 			return nil;
 
 		switch (association->policy) {
 		case OBJC_ASSOCIATION_RETAIN:
 		case OBJC_ASSOCIATION_COPY:
-			ret = [[association->object retain] autorelease];
+			ret = objc_autoreleaseReturnValue(
+			    objc_retain(association->object));
 			break;
 		default:
 			ret = association->object;
@@ -217,7 +240,7 @@ objc_getAssociatedObject(id object, const void *key)
 #ifdef OF_HAVE_THREADS
 	} @finally {
 		if (OFSpinlockUnlock(&spinlocks[slot]) != 0)
-			OBJC_ERROR("Failed to unlock spinlock!");
+			_OBJC_ERROR("Failed to unlock spinlock!");
 	}
 #endif
 
@@ -227,17 +250,29 @@ objc_getAssociatedObject(id object, const void *key)
 void
 objc_removeAssociatedObjects(id object)
 {
-	size_t slot = slotForObject(object);
+	size_t slot;
+
+#if defined(OF_HAVE_ATOMIC_OPS) && defined(OF_OBJFW_RUNTIME)
+	OFReleaseMemoryBarrier();
+
+	if (object != nil && !object_isTaggedPointer(object) &&
+	    (_object_getClass_fast(object)->info &
+	    _OBJC_CLASS_INFO_RUNTIME_RR) && !(_OBJC_PRE_IVARS(object)->info &
+	    _OBJC_OBJECT_INFO_ASSOCIATIONS))
+		return;
+#endif
+
+	slot = slotForObject(object);
 
 #ifdef OF_HAVE_THREADS
 	if (OFSpinlockLock(&spinlocks[slot]) != 0)
-		OBJC_ERROR("Failed to lock spinlock!");
+		_OBJC_ERROR("Failed to lock spinlock!");
 
 	@try {
 #endif
-		objc_hashtable *objectHashtable;
+		_objc_hashtable *objectHashtable;
 
-		objectHashtable = objc_hashtable_get(hashtables[slot], object);
+		objectHashtable = _objc_hashtable_get(hashtables[slot], object);
 		if (objectHashtable == NULL)
 			return;
 
@@ -246,7 +281,7 @@ objc_removeAssociatedObjects(id object)
 			struct Association *association;
 
 			if (objectHashtable->data[i] == NULL ||
-			    objectHashtable->data[i] == &objc_deletedBucket)
+			    objectHashtable->data[i] == &_objc_deletedBucket)
 				continue;
 
 			association = (struct Association *)
@@ -264,7 +299,7 @@ objc_removeAssociatedObjects(id object)
 			case OBJC_ASSOCIATION_RETAIN_NONATOMIC:
 			case OBJC_ASSOCIATION_COPY:
 			case OBJC_ASSOCIATION_COPY_NONATOMIC:
-				[association->object release];
+				objc_release(association->object);
 				break;
 			default:
 				break;
@@ -273,12 +308,12 @@ objc_removeAssociatedObjects(id object)
 			free(association);
 		}
 
-		objc_hashtable_delete(hashtables[slot], object);
-		objc_hashtable_free(objectHashtable);
+		_objc_hashtable_delete(hashtables[slot], object);
+		_objc_hashtable_free(objectHashtable);
 #ifdef OF_HAVE_THREADS
 	} @finally {
 		if (OFSpinlockUnlock(&spinlocks[slot]) != 0)
-			OBJC_ERROR("Failed to unlock spinlock!");
+			_OBJC_ERROR("Failed to unlock spinlock!");
 	}
 #endif
 }
