@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -22,29 +22,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#ifdef OF_OBJFW_RUNTIME
-# import "ObjFWRT.h"
-# import "private.h"
-#else
-# import <objc/runtime.h>
-#endif
+#import "ObjFWRT.h"
+#import "private.h"
 
 #import "macros.h"
 #if !defined(OF_HAVE_COMPILER_TLS) && defined(OF_HAVE_THREADS)
 # import "OFTLSKey.h"
 #endif
 
-#ifndef OF_OBJFW_RUNTIME
-@interface DummyObject
-- (void)release;
-@end
-#endif
-
-#ifndef OBJC_ERROR
-/* This is also used with old Apple runtimes that lack autorelease pools. */
-# define OBJC_ERROR(...) abort()
-#endif
-
 #if defined(OF_HAVE_COMPILER_TLS)
 static thread_local id *objects = NULL;
 static thread_local uintptr_t count = 0;
@@ -62,7 +47,7 @@ OF_CONSTRUCTOR()
 {
 	if (OFTLSKeyNew(&objectsKey) != 0 || OFTLSKeyNew(&countKey) != 0 ||
 	    OFTLSKeyNew(&sizeKey) != 0)
-		OBJC_ERROR("Failed to create TLS keys!");
+		_OBJC_ERROR("Failed to create TLS keys!");
 }
 #endif
 
@@ -91,7 +76,7 @@ objc_autoreleasePoolPop(void *pool)
 	}
 
 	for (uintptr_t i = idx; i < count; i++) {
-		[objects[i] release];
+		objc_release(objects[i]);
 
 #if !defined(OF_HAVE_COMPILER_TLS) && defined(OF_HAVE_THREADS)
 		objects = OFTLSKeyGet(objectsKey);
@@ -109,13 +94,13 @@ objc_autoreleasePoolPop(void *pool)
 #else
 		if (OFTLSKeySet(objectsKey, objects) != 0 ||
 		    OFTLSKeySet(sizeKey, (void *)0) != 0)
-			OBJC_ERROR("Failed to set TLS key!");
+			_OBJC_ERROR("Failed to set TLS key!");
 #endif
 	}
 
 #if !defined(OF_HAVE_COMPILER_TLS) && defined(OF_HAVE_THREADS)
 	if (OFTLSKeySet(countKey, (void *)count) != 0)
-		OBJC_ERROR("Failed to set TLS key!");
+		_OBJC_ERROR("Failed to set TLS key!");
 #endif
 }
 
@@ -135,12 +120,12 @@ _objc_rootAutorelease(id object)
 			size *= 2;
 
 		if ((objects = realloc(objects, size * sizeof(id))) == NULL)
-			OBJC_ERROR("Failed to resize autorelease pool!");
+			_OBJC_ERROR("Failed to resize autorelease pool!");
 
 #if !defined(OF_HAVE_COMPILER_TLS) && defined(OF_HAVE_THREADS)
 		if (OFTLSKeySet(objectsKey, objects) != 0 ||
 		    OFTLSKeySet(sizeKey, (void *)size) != 0)
-			OBJC_ERROR("Failed to set TLS key!");
+			_OBJC_ERROR("Failed to set TLS key!");
 #endif
 	}
 
@@ -148,7 +133,7 @@ _objc_rootAutorelease(id object)
 
 #if !defined(OF_HAVE_COMPILER_TLS) && defined(OF_HAVE_THREADS)
 	if (OFTLSKeySet(countKey, (void *)count) != 0)
-		OBJC_ERROR("Failed to set TLS key!");
+		_OBJC_ERROR("Failed to set TLS key!");
 #endif
 
 	return object;
