@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -91,7 +91,7 @@ test(id self, SEL _cmd)
 - (void)forwardingMessageAndAddingInstanceMethod
 {
 	ForwardingTestObject *testObject =
-	    [[[ForwardingTestObject alloc] init] autorelease];
+	    objc_autorelease([[ForwardingTestObject alloc] init]);
 
 	[testObject test];
 	OTAssertTrue(success);
@@ -105,9 +105,9 @@ test(id self, SEL _cmd)
 - (void)testForwardingTargetForSelector
 {
 	ForwardingTestObject *testObject =
-	    [[[ForwardingTestObject alloc] init] autorelease];
+	    objc_autorelease([[ForwardingTestObject alloc] init]);
 
-	target = [[[ForwardingTarget alloc] init] autorelease];
+	target = objc_autorelease([[ForwardingTarget alloc] init]);
 
 	OTAssertEqual(
 	    [testObject forwardingTargetTest: 0xDEADBEEF
@@ -119,9 +119,9 @@ test(id self, SEL _cmd)
 - (void)testForwardingTargetForSelectorWithVariableArguments
 {
 	ForwardingTestObject *testObject =
-	    [[[ForwardingTestObject alloc] init] autorelease];
+	    objc_autorelease([[ForwardingTestObject alloc] init]);
 
-	target = [[[ForwardingTarget alloc] init] autorelease];
+	target = objc_autorelease([[ForwardingTarget alloc] init]);
 
 	OTAssertEqualObjects(
 	    ([testObject forwardingTargetVarArgTest: FORMAT, ARGS]), RESULT);
@@ -130,15 +130,20 @@ test(id self, SEL _cmd)
 /*
  * Don't try fpret on Win64 if we don't have stret forwarding, as long double
  * is handled as a struct there.
+ *
+ * Don't try fpret on macOS on x86_64 with the Apple runtime as a regression
+ * was introduced in either macOS 14 or 15 where objc_msgSend_fpret calls the
+ * stret forwarding handler instead of the regular one.
  */
-# if !defined(OF_WINDOWS) || !defined(OF_AMD64) || \
-    defined(OF_HAVE_FORWARDING_TARGET_FOR_SELECTOR_STRET)
+# if !(defined(OF_WINDOWS) && defined(OF_AMD64) && \
+    defined(OF_HAVE_FORWARDING_TARGET_FOR_SELECTOR_STRET)) && \
+    !(defined(OF_APPLE_RUNTIME) && defined(OF_AMD64))
 - (void)testForwardingTargetForSelectorFPRet
 {
 	ForwardingTestObject *testObject =
-	    [[[ForwardingTestObject alloc] init] autorelease];
+	    objc_autorelease([[ForwardingTestObject alloc] init]);
 
-	target = [[[ForwardingTarget alloc] init] autorelease];
+	target = objc_autorelease([[ForwardingTarget alloc] init]);
 
 	OTAssertEqual([testObject forwardingTargetFPRetTest],
 	    12345678.00006103515625);
@@ -149,9 +154,9 @@ test(id self, SEL _cmd)
 - (void)testForwardingTargetForSelectorStRet
 {
 	ForwardingTestObject *testObject =
-	    [[[ForwardingTestObject alloc] init] autorelease];
+	    objc_autorelease([[ForwardingTestObject alloc] init]);
 
-	target = [[[ForwardingTarget alloc] init] autorelease];
+	target = objc_autorelease([[ForwardingTarget alloc] init]);
 
 	OTAssertEqual(memcmp([testObject forwardingTargetStRetTest].buffer,
 	    "abcdefghijklmnopqrstuvwxyz", 27), 0);
@@ -161,9 +166,9 @@ test(id self, SEL _cmd)
 - (void)testForwardingTargetForSelectorReturningNilThrows
 {
 	ForwardingTestObject *testObject =
-	    [[[ForwardingTestObject alloc] init] autorelease];
+	    objc_autorelease([[ForwardingTestObject alloc] init]);
 
-	target = [[[ForwardingTarget alloc] init] autorelease];
+	target = objc_autorelease([[ForwardingTarget alloc] init]);
 
 	OTAssertThrowsSpecific([testObject forwardingTargetNilTest],
 	    OFNotImplementedException);
@@ -172,9 +177,9 @@ test(id self, SEL _cmd)
 - (void)testForwardingTargetForSelectorReturningSelfThrows
 {
 	ForwardingTestObject *testObject =
-	    [[[ForwardingTestObject alloc] init] autorelease];
+	    objc_autorelease([[ForwardingTestObject alloc] init]);
 
-	target = [[[ForwardingTarget alloc] init] autorelease];
+	target = objc_autorelease([[ForwardingTarget alloc] init]);
 
 	OTAssertThrowsSpecific([testObject forwardingTargetSelfTest],
 	    OFNotImplementedException);
@@ -184,9 +189,9 @@ test(id self, SEL _cmd)
 - (void)testForwardingTargetForSelectorStRetReturningNilThrows
 {
 	ForwardingTestObject *testObject =
-	    [[[ForwardingTestObject alloc] init] autorelease];
+	    objc_autorelease([[ForwardingTestObject alloc] init]);
 
-	target = [[[ForwardingTarget alloc] init] autorelease];
+	target = objc_autorelease([[ForwardingTarget alloc] init]);
 
 	OTAssertThrowsSpecific([testObject forwardingTargetNilStRetTest],
 	    OFNotImplementedException);
@@ -195,9 +200,9 @@ test(id self, SEL _cmd)
 - (void)testForwardingTargetForSelectorStRetReturningSelfThrows
 {
 	ForwardingTestObject *testObject =
-	    [[[ForwardingTestObject alloc] init] autorelease];
+	    objc_autorelease([[ForwardingTestObject alloc] init]);
 
-	target = [[[ForwardingTarget alloc] init] autorelease];
+	target = objc_autorelease([[ForwardingTarget alloc] init]);
 
 	OTAssertThrowsSpecific([testObject forwardingTargetSelfStRetTest],
 	    OFNotImplementedException);
@@ -272,7 +277,7 @@ test(id self, SEL _cmd)
 				: (double)a2
 				: (double)a3
 {
-	OFEnsure(self == target);
+	OTAssertEqual(self, target);
 
 	if (a0 != (intptr_t)0xDEADBEEF)
 		return 0;
@@ -291,19 +296,18 @@ test(id self, SEL _cmd)
 	va_list args;
 	OFString *ret;
 
-	OFEnsure(self == target);
+	OTAssertEqual(self, target);
 
 	va_start(args, format);
-	ret = [[[OFString alloc] initWithFormat: format
-				      arguments: args] autorelease];
+	ret = [[OFString alloc] initWithFormat: format arguments: args];
 	va_end(args);
 
-	return ret;
+	return objc_autoreleaseReturnValue(ret);
 }
 
 - (long double)forwardingTargetFPRetTest
 {
-	OFEnsure(self == target);
+	OTAssertEqual(self, target);
 
 	return 12345678.00006103515625;
 }
@@ -312,7 +316,7 @@ test(id self, SEL _cmd)
 {
 	struct StretTest ret = { { 0 } };
 
-	OFEnsure(self == target);
+	OTAssertEqual(self, target);
 
 	memcpy(ret.buffer, "abcdefghijklmnopqrstuvwxyz", 27);
 
