@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -38,7 +38,7 @@
 
 - (void)dealloc
 {
-	[_number release];
+	objc_release(_number);
 
 	[super dealloc];
 }
@@ -56,7 +56,12 @@
 
 - (void)testCharValue
 {
-	OTAssertEqual(_number.charValue, 21);
+	OTAssertEqual([[OFNumber numberWithChar: -127] charValue], -127);
+}
+
+- (void)testCharValueOutOfRange
+{
+	OTAssertThrowsSpecific([_number charValue], OFOutOfRangeException);
 }
 
 - (void)testDoubleValue
